
      subroutine axisol(xnose,xend,xblend1,xblend2, Amax, 
     &                  anose, btail, iclose,
     &                  Mach, nc,
     &                  nldim, nl, ilte, xl,zl,sl,dyl,ql)
c============================================================
c     Calculates compressible potential flow about
c     a quasi-axisymmetric body, using a simple 
c     piecewise-constant source line.
c
c   Inputs:
c     xnose     x location of nose point 
c     xend      x location of tail point 
c     xblend1   x location of nose-section blend point
c     xblend2   x location of tail-section blend point
c     Amax      maximum cross-sectional area
c     anose     nose-section shape exponent
c     btail     tail-section shape exponent
c     iclose    if 0, tail tapers to a point, otherwise to an edge
c     Mach      freestream Mach number for Prandtl-Glauert
c     nc        number of control points to be used
c     nldim     max dimension of passed arrays
c
c   Outputs:
c     nl       number of output surface and wake points
c     ilte     index of TE point
c     xl(.)    x locations of surface segment endpoints
c     zl(.)    z locations of surface segment endpoints
c     sl(.)    arc lengths along surface and wake
c     dyl(.)   half-width of edge-type tail section
c     ql(.)    velocities V/V_inf along surface and wake
c
c============================================================
      implicit real (a-h,m,o-z)
      real xl(*), zl(*), sl(*), dyl(*), ql(*)

      parameter (idim=40)
      real xc(idim), zc(idim), ac(idim), dyc(idim), src(idim)
      real zxc(idim), nxc(idim), nyc(idim), nzc(idim)
      real aa(idim,idim), rr(idim)

      data pi / 3.1415926535897932384626 /


ccc   data ispace / 0 /    ! uniform x spacing
      data ispace / 1 /    ! cosine  x spacing


      if(nc .gt. idim) then
       write(*,*)
     &   'AXISOL: Local-array overflow.  Increase idim to', nc
       stop
      endif

      ilte  = nc + 1
      nl = nc + nc/2 + 2

      if(nl .gt. nldim) then
       write(*,*) 
     &   'AXISOL: Passed-in array overflow.  Increase nldim to', nl
       stop
      endif

      beta = sqrt(1.0 - Mach**2)

c---- max radius of equivalent round body
      Rcyl = sqrt(Amax/pi)

c---- set body geometry points
      do i = 1, ilte
        tl = float(i-1) / float(ilte-1)
        if(ispace.eq.0) then
         frac = tl
        else
         frac = 0.5*(1.0 - cos(pi*tl))
        endif
        xl(i) = xnose*(1.0-frac) + xend*frac

        if(i.eq.1 .or. i.eq.ilte) then
         zl(i) = 0.
        elseif(xl(i) .lt. xblend1) then
         f = 1.0 - (xl(i)-xnose)/(xblend1-xnose)
         zl(i) = Rcyl*(1.0 - f**anose)**(1.0/anose)
        elseif(xl(i) .lt. xblend2) then
         zl(i) = Rcyl
        else
         f = (xl(i)-xblend2)/(xend-xblend2)
         zl(i) = Rcyl*(1.0 - f**btail)
        endif

        if(iclose.eq.0) then
          dyl(i) = 0.
        else
          if(xl(i) .lt. xblend2) then
           dyl(i) = 0.
          else
           dyl(i) = Rcyl - zl(i)
          endif
        endif

      enddo
      zl(ilte) = 0.25*zl(ilte-1)
 
      if(iclose.eq.0) then
       dyl(ilte) = 0.
      else
       dyl(ilte) = Rcyl - zl(ilte)
      endif

c---- wake geometry points
      do i = ilte+1, nl
        xl(i) = 2.0*xend - xl(2*ilte-i)
        zl(i) = 0.125*zl(ilte-1)

        if(iclose.eq.0) then
         dyl(i) = 0.
        else
         dyl(i) = Rcyl - zl(i)
        endif
      enddo

c---- calculate arc lengths
      sl(1) = 0.
      do i = 1, nl-1
        ds = sqrt((xl(i+1)-xl(i))**2 + (zl(i+1)-zl(i))**2)
        sl(i+1) = sl(i) + ds
      enddo


c----- calculate wetted area, out of interest
c      Awet = 0.
c      do i = 1, ilte-1
c        ds = sqrt((xl(i+1)-xl(i))**2 + (zl(i+1)-zl(i))**2)
c        ra = 0.5*(zl(i+1)+zl(i))
c        Awet = Awet + 2.0*pi*ra*ds
c      enddo


c---- set control points xc(),zc(.) on actual surface
c-     dyc() is lateral offset over edge-type tail section
      do i = 1, nc
        tlo = float(i-1) / float(ilte-1)
        tlp = float(i  ) / float(ilte-1)

        tc = 0.5*(tlo+tlp)
        if(ispace.eq.0) then
         frac = tc
        else
         frac = 0.5*(1.0 - cos(pi*tc))
        endif
        xc(i) = xnose*(1.0-frac) + xend*frac

cc      xc(i) = 0.5*(xl(i) + xl(i+1))
        if    (xc(i) .lt. xblend1) then
c------- nose section
         f = 1.0 - (xc(i)-xnose)/(xblend1-xnose)
         f_x = -1.0/(xblend1-xnose)
         zc(i) = Rcyl*(1.0 - f**anose)**(1.0/anose)
         z_f = -zc(i)/(1.0 - f**anose) * f**(anose-1.0)
         z_x = z_f*f_x
         dyc(i) = 0.

        elseif(xc(i) .lt. xblend2) then
c------- constant section
         zc(i) = Rcyl
         z_x = 0.
         dyc(i) = 0.

        else
c------- tail section
         f = (xc(i)-xblend2)/(xend-xblend2)
         f_x = 1.0/(xend-xblend2)
         zc(i) = Rcyl*(1.0 - f**btail)
         z_f = -Rcyl*btail*f**(btail-1.0)
         z_x = z_f*f_x

         if(iclose.eq.0) then
c-------- taper to a point
          dyc(i) = 0.
         else
c-------- taper to an edge
          dyc(i) = Rcyl - zc(i)
         endif
        endif

c       dyc(i) = 0.05*zc(i)   !###

c------ surface-normal vector
        nxc(i) =  z_x / sqrt(1.0 + z_x**2)
        nzc(i) = -1.0 / sqrt(1.0 + z_x**2)
        nyc(i) = 0.

        zxc(i) = z_x

c------ meridonal arc length and surface area of i..i+1 segment
        ds = sqrt((xl(i+1)-xl(i))**2 + (zl(i+1)-zl(i))**2)
        ac(i) = ds * 2.0*pi*(zl(i) + 4.0*zc(i) + zl(i+1))/6.0
      enddo


c---- clear AIC matrix
      do i = 1, nc
        rr(i) = 0.
        do j = 1, nc
          aa(i,j) = 0.
        enddo
      enddo

c---- go over control points
      do i = 1, nc
c
c------ go over source segments, setting up V.n=0 contributions at control point i
        yc = 0.
        do j = 1, nc
          if(dyc(j) .eq. 0.0) then
c--------- singularity element is source line over xl(i)...xl(i+1)
           call vline(xc(i),yc,zc(i), xl(j),xl(j+1), u0,v0,w0,beta)
          else
c--------- singularity element is source panel over xl(i)...xl(i+1), y1..y2
           y1 = -dyc(j)
           y2 =  dyc(j)
           call vsurf(xc(i),yc,zc(i), xl(j),xl(j+1),y1,y2,u0,v0,w0,beta)
          endif

c-------- add on contribution to V.n=0 equation
          aa(i,j) = aa(i,j) + u0*nxc(i) + v0*nyc(i) + w0*nzc(i)
        enddo

c------ unit-freestream contribtuion to V.n=0 equation
        rr(i) = nxc(i)
      enddo

c---- solve for all source strenghts
      call gaussn(idim,nc,aa,rr,1)
      do i = 1, nc
        src(i) = -rr(i)
      enddo
      
c      do i = 1, n
c        write(41,*) xc(i), src(i)
c      enddo

c---- calculate velocities at all xl,zl points
      ql(1) = 0.0
      do i = 2, nl
c------ clear velocity accumulators
        ul = 0.
        vl = 0.
        wl = 0.
c
c------ accumulate source veleocities
        yl = 0.
        do j = 1, nc
          if(dyc(j) .eq. 0.0) then
           call vline(xl(i),yl,zl(i), xl(j),xl(j+1), u0,v0,w0,beta)
          else
           y1 = -dyc(j)
           y2 =  dyc(j)
           call vsurf(xl(i),yl,zl(i), xl(j),xl(j+1),y1,y2,u0,v0,w0,beta)
          endif
          ul = ul + u0*src(j)
          vl = vl + v0*src(j)
          wl = wl + w0*src(j)
        enddo
c
c------ add on freestream
        ul = ul + 1.0

c------ set speed
        ql(i) = sqrt(ul**2 + vl**2 + wl**2)

c        write(42,'(1x,9g13.5)') 
c     &     xl(i),zl(i),sl(i), ql(i)
      enddo

      return
      end ! axisol


      subroutine vline(x,y,z,x1,x2, u0,v0,w0,b)
c----------------------------------------------------------
c     Sets velocity u0,v0,w0 at location x,y,z,
c     due to unit-strength source line segment located 
c     at (x1..x2, 0, 0)
c----------------------------------------------------------
c
      data qopi / 0.0795774715459476678 /

      bsq = b**2

      yb = y*b
      zb = z*b

      R1 = sqrt((x1-x)**2 + yb**2 + zb**2)
      R2 = sqrt((x2-x)**2 + yb**2 + zb**2)

      hbsq = yb**2 + zb**2

c      p0_zb = -((x-x2)/R2 - (x-x1)/R1)*qopi/zb
c      w0 = p0_zb*zb_r
c      p0 = log( (R1-x+x1) / (R2-x+x2) ) * qopi / bsq
c      f0 = (R2 - R1 - (x2-x1)) * qopi

      u0 = qopi*(   1.0/R2 -    1.0/R1)         / bsq
      v0 = qopi*((x2-x)/R2 - (x1-x)/R1)*yb/hbsq / b
      w0 = qopi*((x2-x)/R2 - (x1-x)/R1)*zb/hbsq / b

c R(x,y,z,b) = sqrt(x**2 + (y*b)**2 + (z*b)**2)
c ul(x,y,z,b) = ( 1/R(x-x2,y,z,b) - 1/R(x-x1,y,z,b) ) / b**2
c vl(x,y,z,b) = ((x2-x)/R(x-x2,y,z,b) - (x1-x)/R(x-x1,y,z,b))*(y*b)/((y*b)**2 + (z*b)**2) / b
c wl(x,y,z,b) = ((x2-x)/R(x-x2,y,z,b) - (x1-x)/R(x-x1,y,z,b))*(z*b)/((y*b)**2 + (z*b)**2) / b

      return
      end ! vline


      subroutine vsurf(x,y,z, x1,x2,y1,y2, u0,v0,w0, b)
c----------------------------------------------------------
c     Sets velocity u0,v0,w0 at location x,y,z,
c     due to unit-strength source panel segment located
c     at (x1..x2, y1..y2, 0)
c----------------------------------------------------------
      data qopi / 0.0795774715459476678 /

      bsq = b**2

      y1b = y1*b
      y2b = y2*b
      zb = z*b

      R11 = sqrt((x1-x)**2 + y1b**2 + zb**2)
      R12 = sqrt((x1-x)**2 + y2b**2 + zb**2)
      R21 = sqrt((x2-x)**2 + y1b**2 + zb**2)
      R22 = sqrt((x2-x)**2 + y2b**2 + zb**2)

      g11 = log( abs(y1b + R11) )
      g12 = log( abs(y2b + R12) )
      g21 = log( abs(y1b + R21) )
      g22 = log( abs(y2b + R22) )

      h11 = atanh( (x1-x) / R11 )
      h12 = atanh( (x1-x) / R12 )
      h21 = atanh( (x2-x) / R21 )
      h22 = atanh( (x2-x) / R22 )

      t11 = atan2( (x1-x)*y1b , zb*R11 )
      t12 = atan2( (x1-x)*y2b , zb*R12 )
      t21 = atan2( (x2-x)*y1b , zb*R21 )
      t22 = atan2( (x2-x)*y2b , zb*R22 )

      u0 = qopi*(g22 - g21 - g12 + g11)/(y2b-y1b) / bsq
      v0 = qopi*(h22 - h21 - h12 + h11)/(y2b-y1b) / b
      w0 = qopi*(t22 - t21 - t12 + t11)/(y2b-y1b) / b

c gr(x,y,z,xo,yo,b) = log( abs( (yo-y)*b + R(x-xo,y-yo,z,b) ) )
c hy(x,y,z,xo,yo,b) = atanh( (xo-x) / R(x-xo,y-yo,z,b) )
c tz(x,y,z,xo,yo,b) = atan2( (xo-x)*(yo-y)*b , (z*b)*R(x-xo,y-yo,z,b) )

c us(x,y,z,y1,y2,b) = (gr(x,y,z,x2,y2,b) - gr(x,y,z,x2,y1,b) - gr(x,y,z,x1,y2,b) + gr(x,y,z,x1,y1,b)) / (y2-y1) / b**3
c vs(x,y,z,y1,y2,b) = (hy(x,y,z,x2,y2,b) - hy(x,y,z,x2,y1,b) - hy(x,y,z,x1,y2,b) + hy(x,y,z,x1,y1,b)) / (y2-y1) / b**2
c ws(x,y,z,y1,y2,b) = (tz(x,y,z,x2,y2,b) - tz(x,y,z,x2,y1,b) - tz(x,y,z,x1,y2,b) + tz(x,y,z,x1,y1,b)) / (y2-y1) / b**2

      return
      end ! vsurf

