
      subroutine blax(ndim, n,ite, xi,bi,rni, uinv, Reyn, Mach, fexcr,
     &                uei, dsi, thi, tsi, dci, cfi, cdi, cti, hki, phi )
c-----------------------------------------------------------------
c     Axisymmetric boundary layer + wake calculation routine.
c     Uses specified inviscid velocity, corrects for viscous
c     displacement to allow calculation of separated flow.
c
c  Inputs
c  ------
c    ndim    physical array dimension
c    n       number of BL+wake points
c    ite     index of trailing edge point, start of wake
c    xi(.)   arc length array (BL coordinate)
c    bi(.)   lateral width of BL (body perimeter, zero for wake)
c             = 1 for 2D
c    rni(.)  dr/dn
c             = 0 for 2D
c    uinv(.) inviscid velocity
c    Reyn    Reynolds number,  rho_ref u_ref l_ref / mu_ref
c    Mach    Mach number    ,  u_ref / a_ref
c    fexcr   excrescence multiplier, applied to wall Cf 
c             = 1 for smooth wall
c
c  Assumed units for all quantities:
c    l_ref   same unit as used for input xi,bi
c    u_ref   freestream velocity
c    a_ref   freestream speed of sound
c    rho_ref freestream density
c    mu_ref  freestream viscosity
c    
c  Outputs
c  -------
c    uei(i)  edge velocity, ( = uinv(i) + displacement correction )
c    dsi(i)  displacement thickness
c    thi(i)  momentum thickness
c    tsi(i)  kinetic energy thickness
c    dci(i)  density flux thickness
c    cfi(i)  skin friction coefficient, normalized with local rho,u
c    cdi(i)  dissipation coefficient  , normalized with local rho,u
c    cti(i)  max shear-stress coefficient, normalized with local rho,u
c    hki(i)  kinematic shape parameter
c    phi(i)  integrated dissipation
c
c
c  Other outputs of interest can be computed as follows.
c  These are in units of l_ref, rho_ref, u_ref
c
c  Effective perimeter  : beff  =  bi  +  2 pi dsi rni
c  Edge density         : rhi = (1 + 0.5*(gam-1)*Mach^2*(1.0-uei^2))^(1/(gam-1))
c  Total mass defect    : mdef =  rhi uei   dsi beff
c  Total mom. defect    : Pdef =  rhi uei^2 thi beff
c  Total KE defect      : Edef =  rhi uei^3 tsi beff / 2
c  Wall shear force/span: tw b =  rhi uei^2 cfi beff / 2
c  Dissipation integral : Diss =  rhi uei^3 cdi beff
c
c  Body profile drag Dp is the far-downstream momentum defect Pinf,
c  best obtained by applying Squire-Young to the last wake point i = n :
c
c   Pend = rhi*uei^2 * thi * beff
c   Hend = dsi/thi
c   Hinf = 1.0 + (gam-1)*Mach^2
c   Havg = 0.5*(Hend+Hinf)
c   Pinf = Pend * uei^Havg  =  Dp
c
c-----------------------------------------------------------------
      implicit real (a-h,m,o-z)

      real xi(ndim), bi(ndim), rni(ndim), uinv(ndim),
     &     uei(ndim), rhi(ndim),
     &     dsi(ndim), thi(ndim), tsi(ndim), dci(ndim),
     &     cfi(ndim), cdi(ndim), cti(ndim),
     &     hki(ndim), phi(ndim)

      real aa(3,3), bb(3,3), rr(3)

cc      real aa$(3,3), bb$(3,3), rr$(3)


      parameter (idim=60)
      real mdi(idim), uvis(idim)
      real uvis_mdi(idim,idim)
      real dthi(idim), dmdi(idim), duei(idim), ddsi(idim)

      parameter (kdim=3*idim)
      real asys(kdim,kdim), rsys(kdim)

      character*2 s1, s2, s3, s4

      logical simi, lami, wake, direct

c---- pi  and  1/(4 pi)
      data pi   / 3.14159265358979323846264338327950 /
      data qopi / 0.07957747154594766788444188168625718 /
      data gam / 1.4 /

      data hksep / 2.9 /

      data eps / 1.0e-6 /

      if(n .gt. idim) then
       write(*,*) 'BLAX: Local array overflow.  Increase idim to', n
       stop
      endif

      gmi = gam - 1.0

c---- initialize ue to inviscid uinv, and initialize rhoe
      do i = 1, n
        uei(i) = uinv(i)

        trat = 1.0 + 0.5*gmi*Mach**2 * (1.0-uei(i)**2)
        rhi(i) = trat**(1.0/gmi)
      enddo

c---- first point is not calculated if xi=0 there
      if(xi(1) .eq. 0.0) then
       thi(1) = 0.
       dsi(1) = 0.
       mdi(1) = 0.
       phi(i) = 0.
      endif

c=============================================================================
c---- sweep downstream to initialize BL via standard BL march, 
c-     fudging by specifying plausible Hk if separation occurs

      direct = .true.
      do 100 i = 2, n

        simi = xi(i-1) .eq. 0.0

cc      lami = simi     ! laminar   similarity station
        lami = .false.  ! turbulent similarity station

        wake = i  .gt. ite

        x = xi(i)
        b = bi(i)
        rn = rni(i)

        xm = xi(i-1)
        bm = bi(i-1)
        rnm = rni(i-1)

c------ set i-1 station, and initialize station i for iteration
        if(simi) then
         uem = 0.
         thm = 0.
         dsm = 0.

         ue = uei(i)
         rex = ue*x * Reyn
         th = 0.4*x / sqrt(rex)
         ds = th * 2.0

        else
         uem = uei(i-1)
         thm = thi(i-1)
         dsm = dsi(i-1)

         th = thi(i-1)
         ds = dsi(i-1)
         if(direct) then
c-------- previous station was direct... use specified ue as initial guess
          ue = uei(i)
         else
c-------- previous station was inverse... use previous-station ue as initial guess
          ue = uei(i-1)
         endif

        endif
 
c------ always try direct mode first
        direct = .true.
        hkprev = 0.

        do 10 iter = 1, 20
          call blvar(simi,lami,wake, Reyn,Mach, fexcr,
     &                 x, th ,ds ,ue , 
     &                 h , h_th, h_ds,
     &                 hk, hk_th, hk_ds, hk_ue,
     &                 hc, hc_th, hc_ds, hc_ue,
     &                 hs, hs_th, hs_ds, hs_ue,
     &                 cf, cf_th, cf_ds, cf_ue,
     &                 di, di_th, di_ds, di_ue )
          if(iter.eq.1) then
           hkprev = hk
          else
           if(hk .gt. hksep) then ! .and. hk .gt. hkprev) then
c---------- Hk limit exceeded... switch to inverse mode for this point
            direct = .false.
           endif
          endif

c-------- set up 2-point differenced BL equation system
          call blsys(simi,lami,wake,direct, Mach, uinv(i),hksep,
     &                 x,b,rn,th,ds,ue, 
     &                 h , h_th, h_ds,
     &                 hk, hk_th, hk_ds, hk_ue,
     &                 hc, hc_th, hc_ds, hc_ue,
     &                 hs, hs_th, hs_ds, hs_ue,
     &                 cf, cf_th, cf_ds, cf_ue,
     &                 di, di_th, di_ds, di_ue,
     &                 xm,bm,rnm,thm,dsm,uem, 
     &                 hm , hm_thm, hm_dsm,
     &                 hkm, hkm_thm, hkm_dsm, hkm_uem,
     &                 hcm, hcm_thm, hcm_dsm, hcm_uem,
     &                 hsm, hsm_thm, hsm_dsm, hsm_uem,
     &                 cfm, cfm_thm, cfm_dsm, cfm_uem,
     &                 dim, dim_thm, dim_dsm, dim_uem,
     &                 aa,bb,rr)
  
          call gaussn(3,3,aa,rr,1)
          dth = -rr(1)
          dds = -rr(2)
          due = -rr(3)

          rlx = 1.0
          if(rlx*dth .gt.  1.6*th) rlx =  1.6*th/dth
          if(rlx*dth .lt. -0.6*th) rlx = -0.6*th/dth
          if(rlx*dds .gt.  2.5*ds) rlx =  2.5*ds/dds
          if(rlx*dds .lt. -0.4*ds) rlx = -0.4*ds/dds
          if(rlx*due .gt.  0.2*ue) rlx =  0.2*ue/due
          if(rlx*due .lt. -0.1*ue) rlx = -0.1*ue/due

          dmax = max( abs(dth)/th , abs(dds)/ds , abs(due)/ue )

c          if(iter.eq.1) write(*,*)
c          rt = ue*th*Reyn
c          write(*,1200) iter, direct, dmax, rlx, ue, hk, rt, cf, di
c 1200     format(1x,i4, l2, e12.4, 9g13.5)

          th = th + rlx*dth
          ds = ds + rlx*dds
          ue = ue + rlx*due

          if(dmax .lt. eps) go to 15
 10     continue
 15   continue

      uei(i) = ue
      dsi(i) = ds
      thi(i) = th
      tsi(i) = hs*th
      dci(i) = hc*th
      cfi(i) = cf
      cdi(i) = di*hs/2.0
      cti(i) = 0.03 * 0.5*hs*((hk-1.0)/hk)**2
      hki(i) = hk

      dib  = cdi(i)  *rhi(i)  *uei(i)**3   * (b  + 2.0*pi*ds *rn )
      dibm = cdi(i-1)*rhi(i-1)*uei(i-1)**3 * (bm + 2.0*pi*dsm*rnm)
      phi(i) = phi(i-1) + 0.5*(dib + dibm) * (x - xm)
      mdi(i) = ue*ds*(b + 2.0*pi*ds)


      thm = th
      dsm = ds
      hsm = hs
      uem = ue

      hm = h
      hm_thm = h_th
      hm_dsm = h_ds

      hkm = hk
      hkm_thm = hk_th
      hkm_dsm = hk_ds
      hkm_uem = hk_ue

      hsm = hs
      hsm_thm = hs_th
      hsm_dsm = hs_ds
      hsm_uem = hs_ue

      hcm = hc
      hcm_thm = hc_th
      hcm_dsm = hc_ds
      hcm_uem = hc_ue

      dim     = di
      dim_thm = di_th
      dim_dsm = di_ds
      dim_uem = di_ue

      if(i.ge.ite) then
       cfm     = 0.
       cfm_thm = 0.
       cfm_dsm = 0.
       cfm_uem = 0.
      else
       cfm     = cf
       cfm_thm = cf_th
       cfm_dsm = cf_ds
       cfm_uem = cf_ue
      endif

 100  continue


c=============================================================================
c---- perform global viscous/inviscid iteration with source model 
c-     for viscous displacement

c---- size of system
      nsys = 3*n

c---- clear and accumulate source-influence matrix
      do i = 1, n
        do j = 1, n
          uvis_mdi(i,j) = 0.
        enddo
      enddo
      do i = 1, n
        do j = 1, n-1
          dx = xi(i) - 0.5*(xi(j+1)+xi(j))
c         dm = mdi(j+1) - mdi(j)
c         du = dm * qopi / (dx*abs(dx))
c         duei = duei + du
          uvis_mdi(i,j+1) = uvis_mdi(i,j+1) + qopi/(dx*abs(dx))
          uvis_mdi(i,j  ) = uvis_mdi(i,j  ) - qopi/(dx*abs(dx))
        enddo
      enddo

c      i = 15
c      do j = 1, n
c         write(*,*) i, j, uvis_mdi(i,j)
c      enddo

c------------------------------------------------------------
c---- global Newton iteration
      npass = 20
      do 800 ipass = 1, npass

c---- clear system matrix and righthand side
      do k = 1, nsys
        rsys(k) = 0.
        do l = 1, nsys
          asys(k,l) = 0.
        enddo
      enddo

c---- first point variables are held frozen... put 1's on diagonal
      asys(1,1) = 1.0
      asys(2,2) = 1.0
      asys(3,3) = 1.0


c---- set current uvis corresponding to current mdi
      dsi(1) = 0.
      do i = 2, n
        uvis(i) = uinv(i)
        do j = 1, n
          uvis(i) = uvis(i) + uvis_mdi(i,j)*mdi(j)
        enddo
      enddo

c---- sweep downstream to set up BL equations
      do 200 i = 2, n
        simi = xi(i-1) .eq. 0.0

cc      lami = simi
        lami = .false.

        wake = i  .gt. ite

        x  = xi(i)
        b  = bi(i)
        rn = rni(i)
        th = thi(i)
        md = mdi(i)
        ue = uei(i)

        xm  = xi(i-1)
        bm  = bi(i-1)
        rnm = rni(i-1)
        thm = thi(i-1)
        mdm = mdi(i-1)
        uem = uei(i-1)

cc      md = ue*ds*(b + 2.0*pi*ds*rn)

cc      md/(ue*2*pi) = ds*(b/2pi + ds*rn)
cc      0.5 rn ds^2 + (b/4pi)*ds - md/(4pi ue) = 0
cc      ds = ( sqrt(bp^2 + 0.5*md/(pi*ue)) - bp ) / rn
        bp = b * 0.25/pi
        mpu = 0.5*md/(pi*ue)
        if(rn .le. 1.0e-6) then
         ds    =  md/(ue*b)
         ds_md = 1.0/(ue*b)
         ds_ue = -ds/ue
        else
         ds    =    (sqrt(bp**2 + mpu) - bp) / rn
         ds_md = 0.5/sqrt(bp**2 + mpu) * ( mpu/md) / rn
         ds_ue = 0.5/sqrt(bp**2 + mpu) * (-mpu/ue) / rn
        endif

        if(simi) then
         dsm = 0.
         dsm_mdm = 0.
         dsm_uem = 0.
        else
         if(rnm .le. 1.0e-6) then
          dsm     =  mdm/(uem*bm)
          dsm_mdm = 1.0/(uem*bm)
          dsm_uem = -dsm/uem
         else
          bpm = bm * 0.25/pi
          mpum = 0.5*mdm/(pi*uem)
          dsm     =    (sqrt(bpm**2 + mpum) - bpm) / rnm
          dsm_mdm = 0.5/sqrt(bpm**2 + mpum) * ( mpum/mdm) / rnm
          dsm_uem = 0.5/sqrt(bpm**2 + mpum) * (-mpum/uem) / rnm
         endif
        endif

c        if(.not.simi) then
c        call blvar(simi,lami,wake, Reyn,Mach, fexcr,
c     &                 xm, thm ,dsm ,uem , 
c     &                 hm, hm_thm, hm_dsm,
c     &                 hkm, hkm_thm, hkm_dsm, hkm_uem,
c     &                 hcm, hcm_thm, hcm_dsm, hcm_uem,
c     &                 hsm, hsm_thm, hsm_dsm, hsm_uem,
c     &                 cfm, cfm_thm, cfm_dsm, cfm_uem,
c     &                 dim, dim_thm, dim_dsm, dim_uem )
c        endif
        call blvar(simi,lami,wake, Reyn,Mach, fexcr,
     &                 x, th ,ds ,ue , 
     &                 h , h_th, h_ds,
     &                 hk, hk_th, hk_ds, hk_ue,
     &                 hc, hc_th, hc_ds, hc_ue,
     &                 hs, hs_th, hs_ds, hs_ue,
     &                 cf, cf_th, cf_ds, cf_ue,
     &                 di, di_th, di_ds, di_ue )
        direct = .true.
        call blsys(simi,lami,wake,direct, Mach, uinv(i),hksep,
     &                 x,b,rn,th,ds,ue, 
     &                 h , h_th, h_ds,
     &                 hk, hk_th, hk_ds, hk_ue,
     &                 hc, hc_th, hc_ds, hc_ue,
     &                 hs, hs_th, hs_ds, hs_ue,
     &                 cf, cf_th, cf_ds, cf_ue,
     &                 di, di_th, di_ds, di_ue,
     &                 xm,bm,rnm,thm,dsm,uem, 
     &                 hm , hm_thm, hm_dsm,
     &                 hkm, hkm_thm, hkm_dsm, hkm_uem,
     &                 hcm, hcm_thm, hcm_dsm, hcm_uem,
     &                 hsm, hsm_thm, hsm_dsm, hsm_uem,
     &                 cfm, cfm_thm, cfm_dsm, cfm_uem,
     &                 dim, dim_thm, dim_dsm, dim_uem,
     &                 aa,bb,rr)

c------ put BL equations of small 3x3 system into big system
        do k = 1, 2
          ksys = 3*(i-1) + k
          rsys(ksys) = rr(k)

          r_th = aa(k,1)
          r_ds = aa(k,2)
          r_ue = aa(k,3)

          r_thm = bb(k,1)
          r_dsm = bb(k,2)
          r_uem = bb(k,3)

          lsys = 3*(i-1) + 1
          asys(ksys,lsys  ) = r_th
          asys(ksys,lsys-3) = r_thm

          lsys = 3*(i-1) + 2
          asys(ksys,lsys  ) = r_ds *ds_md
          asys(ksys,lsys-3) = r_dsm*dsm_mdm

          lsys = 3*(i-1) + 3
          asys(ksys,lsys  ) = r_ds *ds_ue   + r_ue
          asys(ksys,lsys-3) = r_dsm*dsm_uem + r_uem
        enddo

c------ set up 3rd ue equation, ue = uvis
        k = 3
        ksys = 3*(i-1) + k
        rsys(ksys) = ue - uvis(i)

        do j = 1, n
          lsys = 3*(j-1) + 2
          asys(ksys,lsys) = -uvis_mdi(i,j)
        enddo

        lsys = 3*(i-1) + 3
        asys(ksys,lsys) = asys(ksys,lsys) + 1.0

c------ also store dependent variables for returning
        tsi(i) = hs*th
        dci(i) = hc*th
        cfi(i) = cf
        cdi(i) = di*hs/2.0
        cti(i) = 0.03 * 0.5*hs*((hk-1.0)/hk)**2
        hki(i) = hk

c------ set dependent i-1 variables for next interval
        hm = h
        hm_thm = h_th
        hm_dsm = h_ds
  
        hkm = hk
        hkm_thm = hk_th
        hkm_dsm = hk_ds
        hkm_uem = hk_ue
  
        hsm = hs
        hsm_thm = hs_th
        hsm_dsm = hs_ds
        hsm_uem = hs_ue
  
        hcm = hc
        hcm_thm = hc_th
        hcm_dsm = hc_ds
        hcm_uem = hc_ue
  
        if(i.ge.ite) then
         cfm     = 0.
         cfm_thm = 0.
         cfm_dsm = 0.
         cfm_uem = 0.
        else
         cfm     = cf
         cfm_thm = cf_th
         cfm_dsm = cf_ds
         cfm_uem = cf_ue
        endif
  
        dim     = di
        dim_thm = di_th
        dim_dsm = di_ds
        dim_uem = di_ue

 200  continue


c      do i = 1, n
c        k1 = 3*(i-1) + 1
c        k2 = 3*(i-1) + 2
c        k3 = 3*(i-1) + 3
c        write(*,'(1x,i4,3e12.4,3x,3g15.7)')
c     &    i, rsys(k1),rsys(k2),rsys(k3), uei(i), uvis(i), uei(i)-uvis(i)
c      enddo

c---- solve Newton system
      call gaussn(kdim,nsys,asys,rsys,1)

c---- set Newton changes, set max change dmax, and set limiter rlx
        dmax = 0.
        rlx = 1.0
        do i = 2, n
          b  = bi(i)
          th = thi(i)
          md = mdi(i)
          ue = uei(i)

          kth = 3*(i-1) + 1
          kmd = 3*(i-1) + 2
          kue = 3*(i-1) + 3 
          dthi(i) = -rsys(kth)
          dmdi(i) = -rsys(kmd)
          duei(i) = -rsys(kue)

          bp = b * 0.25/pi
          mpu = 0.5*md/(pi*ue)
          ds    =     sqrt(bp**2 + mpu) - bp
          ds_md = 0.5/sqrt(bp**2 + mpu) * ( mpu/md)
          ds_ue = 0.5/sqrt(bp**2 + mpu) * (-mpu/ue)

c          ds    =  md/(b*ue)
c          ds_md = 1.0/(b*ue)
c          ds_ue = -ds/ue

          ddsi(i) = ds_md*dmdi(i) + ds_ue*duei(i)

          if(rlx*dthi(i) .gt.  1.6*th) rlx =  1.6*th/dthi(i)
          if(rlx*dthi(i) .lt. -0.6*th) rlx = -0.6*th/dthi(i)
          if(rlx*ddsi(i) .gt.  2.5*ds) rlx =  2.5*ds/ddsi(i)
          if(rlx*ddsi(i) .lt. -0.4*ds) rlx = -0.4*ds/ddsi(i)
          if(rlx*duei(i) .gt.  0.2*ue) rlx =  0.2*ue/duei(i)
          if(rlx*duei(i) .lt. -0.1*ue) rlx = -0.1*ue/duei(i)

          dmax = max( dmax , abs(dthi(i))/th , 
     &                       abs(ddsi(i))/ds , 
     &                       abs(duei(i))/ue   )
        enddo

c------ perform Newton updated, with limiter if rlx < 1
        do i = 2, n
          thi(i) = thi(i) + rlx*dthi(i)
          dsi(i) = dsi(i) + rlx*ddsi(i)
          uei(i) = uei(i) + rlx*duei(i)
          mdi(i) = uei(i)*dsi(i)*(bi(i) + 2.0*pi*dsi(i)*rni(i))
        enddo

c        do i = 2, n
c          thi(i) = thi(i) + rlx*dthi(i)
ccc        dsi(i) = dsi(i) + rlx*ddsi(i)
c          uei(i) = uei(i) + rlx*duei(i)
c          mdi(i) = mdi(i) + rlx*dmdi(i)
c          dsi(i) = mdi(i)/(bi(i)*uei(i))
c        enddo

c        if(ipass.eq.1) write(*,*)
c        write(*,2200) ipass, dmax, rlx
c 2200   format(1x,i4, e12.4, f8.4)

        if(dmax .lt. eps) go to 810

 800  continue
 810  continue


c---- integrate running dissipation
      phi(1) = 0.
      do i = 2, n
        x  = xi(i)
        xm = xi(i-1)

        dib  = cdi(i)  *rhi(i)  *uei(i)**3  
     &       * (bi(i)   + 2.0*pi*dsi(i)  *rni(i))
        dibm = cdi(i-1)*rhi(i-1)*uei(i-1)**3
     &       * (bi(i-1) + 2.0*pi*dsi(i-1)*rni(i-1))

        phi(i) = phi(i-1) + 0.5*(dib + dibm) * (x - xm)
      enddo

      return
      end ! blax
