
      subroutine simpop(ndim,n, dxref,x,f,
     &                  ftol,fdel,istep0,istepn,Lopconv)
c----------------------------------------------------------------------
C   Multidimensional minimization routine of the function supplied 
C   by user in FOBJ(n,x,f), where x is an n-long vector and f is 
C   the functional value at x. That is, f(x).
C
C Input:
C   ndim      Physical first dimension of x(..) array
C   n         Number of degrees of freedom (size of x(.*))
C   dxref(.)  Expected typical x(.) change, used for scaling
C   x(.*)     Initial starting simplex, whose n+1 columns are the 
C               simplex vertices, or points in the design space.
C             The points must be distinct (no duplicate columns)
C   ftol      convergence tolerance to be achieved in the function value
C   istep0    index of last optimization step previously taken (0 if none)
C   istepn    index of last optimization step to be taken here
C
C Output:
C   x(.*)     Modified simplex, whose columns are improved 
C               points in the design space
C   f(*) Function values at the vertex points.
C
C             Best returned point is x(.1) 
C              with minimum function f(1)
C
C
C   This routine calls subroutine fobj, 
C   which returns the objective function value:
C
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CC      subroutine fobj(n,x,f,dfrel,istep)
CC
CC      Input:
CC        n        Size of x, same as n passed into simplex
CC        x(.)     Variable vector
CC        dfrel    Relative error so far,  convergence when dfrel < 1
CC        istep    Optimization step index (may or may not be useful)
CC
CC       Output:
CC        f        Value of function for passed-in variables x(.)
CC
CC
CC    Note:  dfrel and ivprint are used only for info printout purposes.
CC           dfrel can be written without them, e.g.
CC
CC              subroutine fobj(n,x,f)
CC
CC           in which case they must also be removed from the 
CC              call fobj(...) statements here.
CC
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C   Method uses the downhill simplex method of Nelder and Mead
C   (Computer Journal, 7, 308 (1965). Code contains elements from
C   Numerical Recipes by Press et al. and code written by Gary Sowers
C   (Univ. of Deleware). Modified for ChEg 698D by E. Maginn, Univ. of
C   Notre Dame, Tue Jan 25, 2000 10:33PM..
C   Please report bugs to ed@nd.edu.
C
C	  The program has been slightly modified for MAE207 by G. Arya of 
C	  UC San Diego on Thu Oct 9 2008
C
c----------------------------------------------------------------------
      implicit none
c----------------------------------------------------------------------
c---- passed variables
      integer ndim, n, istep0, istepn
      logical Lopconv
      real dxref(*), x(ndim,*), f(*), ftol, fdel
c
c----------------------------------------------------------------------
c---- local variables
      integer nmax
      parameter (nmax=21)
      real xrfl(nmax),xexp(nmax)
      real xcon(nmax),xcen(nmax)
      real g(nmax)
      real fx(nmax)
      real ftemp(nmax), xtemp(nmax,nmax)

      integer i,j,np1
      integer istep, ist, jlo,jhi,jnxhi
      integer ng, ifxmax
      integer jsort(nmax)
      integer lu

      real dfrel,fexp,frfl,fcon
      real temp, fun

      real fxmax, fxrms, fxnorm

      real xhi(21), xlo(21)
      real fhi, flo

c----------------------------------------------------------------------
c---- simplex change parameters
C       alpha   size of reflections
C       beta    size of contractions,  < 1
C       gamma   size of expansions  ,  > 1
C
      real alpha,beta,gamma
c     data  alpha, beta, gamma
c    &    / 1.0  , 0.5  , 2.0  /
c     data  alpha, beta, gamma
c    &    / 1.0  , 0.6  , 1.6  /
      data  alpha, beta, gamma
     &    / 1.0  , 0.7  , 1.4  /
c
      if(n+1 .gt. nmax) then
       write(*,*) 'SIMPOP: Local array overflow. Increase nmax to', n+1
       stop
      endif
c
c----------------------------------------------------------------------
c
c---- number of vertices ( x(.*) columns )
      np1 = n + 1

      if(istep0 .eq. 0) then
c----- set initial function values at all simplex vertices
       do j = 1, np1
         if(j.eq.1) then
          ist = 1   ! print header on first simplex point output
         else
          ist = 0
         endif
         call fobj(n,x(1,j),
     &             ng,g, fun,
     &             f(j),0.0,'j',ist)
       enddo
      endif


c======================================================================
c---- top of minimization loop
      do 100 istep = istep0+1, istepn

c---- sort simplex points, from lowest to highest function values
      call hsort(np1,f,jsort)

      do j = 1, np1
        ftemp(j) = f(j)
        do i = 1, n
          xtemp(i,j) = x(i,j)
        enddo
      enddo

      do j = 1, np1
        f(j) = ftemp(jsort(j))
        do i = 1, n
          x(i,j) = xtemp(i,jsort(j))
        enddo
      enddo

      jlo = 1
      jnxhi = np1-1
      jhi = np1

c      lu = 40
c      do j = 1, np1
c        write(lu+j,*) istep, f(j)/4.44822, (x(i,j), i=1, n)
c      enddo
c
c      call gsimp(ndim,n,x,f,fx)
c      fxmax = 0.
c      fxrms = 0.
c      ifxmax = 0.
c      do i = 1, n
c        fxnorm = fx(i)*dxref(i)
c        if(abs(fxnorm) .gt. abs(fxmax) ) then
c         ifxmax = i
c         fxmax = fxnorm
c        endif
c        fxrms = fxrms + fxnorm**2
c      enddo
c      fxrms = sqrt( fxrms/float(n) )


c----------------------------------------------------------------------
c---- Check for convergence
      fdel =  f(jhi) - f(jlo)
      dfrel = fdel/ftol

      if (dfrel .lt. 1.0) then
c       write(*,*) "istep = ", istep
c       write(*,*) 'Converged normally'
       Lopconv = .true.
       goto 990
      endif

c----------------------------------------------------------------------
c---- Now, with the best and worst points picked out, we calculate the
c---- centroid of the simplex. Through this point, the simplex will be
c---- expanded and contracted and it "slithers" around searching for
c---- a minimum. This is done by exploring along the ray from the high
c---- point through the center.

c---- Compute the vector average of all points except the highest,
c---- i.e. the center of the "face" across from the highest point. 
      do i = 1, n
        xcen(i) = 0.
      enddo
      do j = 1, np1
        if (j .ne. jhi) then
          do i = 1, n
            xcen(i) = xcen(i) + x(i,j)
          enddo
        endif
      enddo
      do i = 1, n
        xcen(i) = xcen(i) / float(n)
      enddo

c---- Reflect the worst point through the centroid of the simplex
c-     by an amount alpha. 
      do i = 1, n
        xrfl(i) = (1.0+alpha)*xcen(i) - alpha*x(i,jhi)
      enddo

c---- Evaluate the function at the reflected point
      call fobj(n,xrfl,
     &          ng,g, fun,
     &          frfl,dfrel,'r',istep)

c----------------------------------------------------------------------
c---- See how it compares to the best previous point
      if (frfl .le. f(jlo)) then
c----   Reflected point is better than the previous lowest... 
c-       try additional extrapolation by gamma
        do i = 1, n
          xexp(i) = gamma*xrfl(i) + (1.0 - gamma)*xcen(i)
        enddo
        call fobj(n,xexp,
     &            ng,g, fun,
     &            fexp,dfrel,'e',istep)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
        if (fexp .lt. f(jlo)) then
c-------- Expansion was successful... 
c-         replace worst point with the expanded point
          do i = 1, n
            x(i,jhi) = xexp(i)
          enddo
          f(jhi) = fexp

        else
c-------- Expansion didn't improve things...
c-         so use the reflected point instead (ignore the expansion)
          do i = 1, n
            x(i,jhi) = xrfl(i)
          enddo
          f(jhi) = frfl
        endif

c----------------------------------------------------------------------
c---- See how it compares with 2nd worst point
      elseif (frfl .ge. f(jnxhi)) then
c------ The reflected point is worse than the 2nd worst point...

        if (frfl .lt. f(jhi)) then
c-------- But it's better than worst point, so use it to replace worst point
          do i = 1, n
            x(i,jhi) = xrfl(i)
          enddo
          f(jhi) = frfl
        endif

c------ Contract the simplex looking for an intermediate better point
        do i = 1, n
          xcon(i) = beta*x(i,jhi) + (1.0-beta)*xcen(i)
        enddo

c------ Evaluate function at contracted point
        call fobj(n,xcon,
     &            ng,g, fun,
     &            fcon,dfrel,'c',istep)
c--------------------------------------------------------------------
c------ Check to see if contraction is an improvement over worst point
        if (fcon .lt. f(jhi)) then
c-------- It is an improvement, so assign it as the new worst point
          do i = 1, n
            x(i,jhi) = xcon(i)
          enddo
          f(jhi) = fcon

        else
c-------- The contraction is worse than all current points,
c-         so contract around the best point. That is, move
c-         the entire simplex in the direction of the best point.
          do j = 1, np1
            if (j .ne. jlo) then
              do i = 1, n
                x(i,j) = beta*x(i,j) + (1.0-beta)*x(i,jlo)
              enddo

              call fobj(n,x(1,j),
     &                  ng,g, fun,
     &                  f(j),dfrel,'m',istep)
            endif
          enddo

        endif

c--------------------------------------------------------------------
      else
c------ Reflected point is between 2nd-worst and best points
c-      Put the reflected point into the worst-point slot, and keep going
c-       (the upcoming sort with rearrange all these points)
        do i = 1, n
          x(i,jhi) = xrfl(i)
        enddo
        f(jhi) = frfl

      endif   

 100  continue  ! with next step

      fdel = f(jhi) - f(jlo)
      dfrel = fdel/ftol
      write(*,'(1x,a,g12.4)')
     &   'SIMPOP: Exiting on step limit.  dFrel =', dfrel
      Lopconv = .false.


c---- pick up here on convergence
 990  continue

c---- return minimum-function vector in first column x(i,1)
      if(jlo .ne. 1) then
c----- swap columns 1 and jlo
       do i = 1, n
         temp = x(i,jlo)
         x(i,jlo) = x(i,1)
         x(i,1) = temp
       enddo
       temp = f(jlo)
       f(jlo) = f(1)
       f(1) = temp

       if(jhi.eq.1) jhi = jlo
       if(jnxhi.eq.1) jnxhi = jlo
       jlo = 1
      endif

c---- final minimum function value
      call fobj(n,x(1,1),
     &          ng,g, fun,
     &          f(1),dfrel,'!',istep)
c      write(*,*) 'Nstep =', istep

      return
      end ! simpop


      subroutine gsimp(ndim,n,x,f, fx)
      implicit real (a-h,o-z)
      real x(ndim,*), f(*)
      real fx(*)

      parameter (nmax = 21)
      real sys(nmax,nmax), res(nmax)

      if(n .gt. nmax) then
       write(*,*) 'GSIMP: Array overflow.  Increase nmax to', n
       stop
      endif

      do i = 1, n
        do j = 1, n
          sys(i,j) = 0.
        enddo

        fx(i) = f(i+1) - f(1)
        do j = 1, n
          sys(i,j) = sys(i,j) + (x(i,j+1) - x(i,1))
        enddo
      enddo

      call gaussn(nmax,n,sys,fx,1)

      return
      end ! gsimp
