
      subroutine simpop(ndim,n, simp,fvertx,
     &                  ftol,fdel,nstepmax)
c----------------------------------------------------------------------
C   Multidimensional minimization routine of the function supplied 
C   by user in FOBJ(n,x,f), where x is an n-long vector and f is 
C   the functional value at x. That is, f(x).
C
C Input:
C   ndim      Physical first dimension of simp(..) array
C   n         Number of degrees of freedom (size of simp(.*))
C   simp(.*)  Initial starting simplex, whose n+1 columns are the 
C               simplex vertices, or points in the design space.
C             The points must be distinct (no duplicate columns)
C   ftol      convergence tolerance to be achieved in the function value
C   nstepmax  max number of optimization steps simpop is allowed to make
C
C Output:
C   simp(.*)  Modified simplex, whose columns are improved 
C              points in the design space
C   fvertx(*) Function values at the vertex points.
C
C             Best returned point is simp(.1) 
C              with minimum function fvertx(1)
C
C
C   This routine calls subroutine fobj, 
C   which returns the objective function value:
C
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CC      subroutine fobj(nvar,var,fun,dfrel,istep)
CC
CC      Input:
CC        nvars    Size of var, same as n passed into simplex
CC        var(.)   Variable vector
CC        dfrel    Relative error so far,  convergence when dfrel < 1
CC        istep    Optimization step index (may or may not be useful)
CC
CC       Output:
CC        fun      Value of function for passed-in variables var(.)
CC
CC
CC    Note:  dfrel and ivprint are used only for info printout purposes.
CC           dfrel can be written without them, e.g.
CC
CC              subroutine fobj(nvar,var,fun)
CC
CC           in which case they must also be removed from the 
CC              call fobj(...) statements here.
CC
C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C
C   Method uses the downhill simplex method of Nelder and Mead
C   (Computer Journal, 7, 308 (1965). Code contains elements from
C   Numerical Recipes by Press et al. and code written by Gary Sowers
C   (Univ. of Deleware). Modified for ChEg 698D by E. Maginn, Univ. of
C   Notre Dame, Tue Jan 25, 2000 10:33PM..
C   Please report bugs to ed@nd.edu.
C
C	  The program has been slightly modified for MAE207 by G. Arya of 
C	  UC San Diego on Thu Oct 9 2008
C
c----------------------------------------------------------------------
      implicit none
c----------------------------------------------------------------------
c---- passed variables
      integer ndim, n, nstepmax
      real simp(ndim,*), fvertx(*), ftol, fdel
c
c----------------------------------------------------------------------
c---- local variables
      integer nmax
      parameter (nmax=21)
      real preflc(nmax),pexpan(nmax)
      real pcontr(nmax),pcentr(nmax)
      real fcon(nmax)

      integer i,j,np1
      integer istep, jlow,jhigh,jnexthi
      integer nfcon

      real dfrel,fexpan,freflc,fcontr
      real temp, fun
c
c----------------------------------------------------------------------
c---- simplex change parameters
C       alpha   size of reflections
C       beta    size of contractions,  < 1
C       gamma   size of expansions  ,  > 1
C
      real alpha,beta,gamma
c     data  alpha, beta, gamma
c    &    / 1.0  , 0.5 , 2.0  /
      data  alpha, beta, gamma
     &    / 1.0  , 0.7 , 1.4  /
c
      if(n .gt. nmax) then
       write(*,*) 'SIMPOP: Local array overflow. Increase nmax to', n
       stop
      endif
c
c----------------------------------------------------------------------
c
c---- number of vertices ( simp(.*) columns )
      np1 = n + 1

c      subroutine fobj(nvopt,vopt,
c     &                nfcon,fcon, fun,func, dfrel, ch,istep)

c---- set initial function values at simplex vertices
      do j = 1, np1
        call fobj(n,simp(1,j),
     &            nfcon,fcon, fun,
     &            fvertx(j),0.0,'j',0)
      enddo
c
c======================================================================
c---- top of minimization loop
      do 100 istep = 1, nstepmax

c---- Determine the highest, next highest and lowest vertex points.
c---- Initialize...
      jlow = 1
      if (fvertx(1) .gt. fvertx(2)) then 
        jhigh  = 1
        jnexthi = 2                      
      else
        jhigh  = 2
        jnexthi = 1
      endif

c---- loop over all points of the simplex, and sort according to highest
c---- next highest and lowest points.
      do j = 1, np1               
        if (fvertx(j) .lt. fvertx(jlow) ) then
          jlow = j
ccc        WRITE(50,4441) istep, jlow, (simp(i,jlow),i=1,n), fvertx(jlow)
        endif

        if ( fvertx(j) .gt. fvertx(jhigh) ) then
          jnexthi = jhigh
          jhigh  = j
        elseif (fvertx(j) .gt. fvertx(jnexthi)) then
          if (j .ne. jhigh) jnexthi = j
        endif

      enddo

c----------------------------------------------------------------------
c---- compute the range from highest to lowest 
      fdel = ABS(fvertx(jhigh)-fvertx(jlow))
C      write(*,*) fdel, fvertx(jhigh),fvertx(jlow)
      dfrel = fdel/ftol
c---- Check to see if we continue
      if (dfrel .lt. 1.0) then
c        write(*,*) "istep = ", istep
c        write(*,*) 'Converged normally'
        goto 998
      endif

c----------------------------------------------------------------------
c---- Now, with the best and worst points picked out, we calculate the
c---- centroid of the simplex. Through this point, the simplex will be
c---- expanded and contracted and it "slithers" around searching for
c---- a minimum. This is done by exploring along the ray from the high
c---- point through the center.

c---- zero all counters
      do i = 1, n
        pcentr(i) = 0.
      enddo

c---- Here, we compute the vector average of all points except the
c---- highest. That is, the center of the "face" across from the highest
c---- point. 
      do j = 1, np1
        if (j .ne. jhigh) then
          do i = 1, n
            pcentr(i) = pcentr(i) + simp(i,j)
          enddo
        endif
      enddo

      do i = 1, n
        pcentr(i) = pcentr(i) / float(n)

c----   Reflect the worst point through the centroid of the simplex by
c----   an amount alpha. 
        preflc(i) = (1.0+alpha)*pcentr(i) - alpha*simp(i,jhigh)
      enddo

c----------------------------------------------------------------------
c---- Evaluate the function at the reflected point
      call fobj(n,preflc,
     &          nfcon,fcon, fun,
     &          freflc,dfrel,'r',istep)

c      if(istep .gt. 600) then
c      write(20,4444) jlow, jnexthi, jhigh, 
c     &     fvertx(jlow)   -153420.0, 
c     &     fvertx(jnexthi)-153420.0, 
c     &     fvertx(jhigh)  -153420.0, 
c     &     freflc         -153420.0
c 4444 format(1x,3i3, 20f9.5)
c      do i = 1, n
c        write(20+i,4440) 
c     &     simp(i,jlow), 
c     &     simp(i,jnexthi),
c     &     simp(i,jhigh),
c     &     preflc(i),
c     &     pcentr(i)
c      enddo
c 4440 format(1x, 20g18.10)
c 4441 format(1x, i5, i3, 20g20.12)
c      endif

c---- See how it compares ot the best previous point
      if (freflc .le. fvertx(jlow)) then
c----   Gives a better point than the previous lowest. So try
c----   additional extrapolation by gamma
        do i = 1, n
          pexpan(i) = gamma*preflc(i) + (1.0 - gamma)*pcentr(i)
        enddo

c----   evaluate the function at the expanded point
        call fobj(n,pexpan,
     &            nfcon,fcon, fun,
     &            fexpan,dfrel,'e',istep)

c--------------------------------------------------------------------
c----   if the expansion was successful, replace the previous worst
c----   vertex with the expanded point
        if (fexpan .lt. fvertx(jlow)) then
          do i = 1, n
            simp(i,jhigh) = pexpan(i)
          enddo
          fvertx(jhigh) = fexpan
        else
c----     otherwise, the expansion by gamma didn't improve
c----     things. Still use the reflected point, since it was better
c----     than the previous best vertex
          do i = 1, n
            simp(i,jhigh) = preflc(i)
          enddo
          fvertx(jhigh) = freflc
        endif
c--------------------------------------------------------------------
c----   else if the reflected point is actually worse than the 2nd
c----   worst vertex...
      elseif (freflc .ge. fvertx(jnexthi)) then

c----   check to see if it is at least better than the worst point,
c----   and replace the worst point with the reflected point if this
c----   is the case.
        
c--------------------------------------------------------------------
        if (freflc .lt. fvertx(jhigh)) then
c----     The reflected point is better than worst point, so replace
c----     worst point.
          do i = 1, n
            simp(i,jhigh) = preflc(i)
          enddo
          fvertx(jhigh) = freflc
        endif
c--------------------------------------------------------------------
c----   Contract the simplex looking for an intermediate lower point
        do i = 1, n
          pcontr(i) = beta*simp(i,jhigh) + (1.0-beta)*pcentr(i)
        enddo

c----   evaluate function at contracted point
        call fobj(n,pcontr,
     &            nfcon,fcon, fun,
     &            fcontr,dfrel,'c',istep)
c--------------------------------------------------------------------
c----   check to see if contraction is an improvement over worst point
        if (fcontr .lt. fvertx(jhigh)) then
c----     it is an improvement, so assign it as the new worst point
          do i = 1, n
            simp(i,jhigh) = pcontr(i)
          enddo
          fvertx(jhigh) = fcontr
        else
c----     else the contraction is worse than all current vertices. In
c----     this case, contract around the best vertex. That is, move
c----     the entire simplex in the direction of the best point.

          do j = 1, np1
            if (j .ne. jlow) then
              do i = 1, n
                pcontr(i) = 0.5*(simp(i,j) + simp(i,jlow))
                simp(i,j) = pcontr(i)
              enddo

              call fobj(n,pcontr,
     &                  nfcon,fcon, fun,
     &                  fvertx(j),dfrel,'m',istep)
            endif
          enddo

        endif
c--------------------------------------------------------------------
      else
c----   arrive here if the reflected point gives a function value
c----   intermediate between the 2nd worst and the best. Call the
c----   reflected point worst and move on.

        do i = 1, n
          simp(i,jhigh) = preflc(i)
        enddo
        fvertx(jhigh) = freflc

      endif                   ! if (freflc .le. fvertx(jlow)) then

c      if(mod(istep,25) .eq. 0) then
cc----- recalculate lowest point, to avoid hysteresis problems
c       temp = fvertx(jlow)
c       call fobj(n,simp(1,jlow),
c     &           nfcon,fcon, fun,
c     &           fvertx(jlow),dfrel,'x',istep)
c       write(*,*) temp
c       write(*,*) fvertx(jlow)
c      endif

 100  continue  ! with next step

      fdel = ABS(fvertx(jhigh)-fvertx(jlow))
      dfrel = fdel/ftol
      write(*,'(1x,a,g12.4)')
     &   'SIMPOP: Exiting on step limit.  dFrel =', dfrel

 998  continue

c---- return minimmum-function vector in first column simp(i,1)
      if(jlow .ne. 1) then
c----- swap columns 1 and jlow
       do i = 1, n
         temp = simp(i,jlow)
         simp(i,jlow) = simp(i,1)
         simp(i,1) = temp
       enddo
       temp = fvertx(jlow)
       fvertx(jlow) = fvertx(1)
       fvertx(1) = temp
      endif

c---- final minimum function value
      call fobj(n,simp(1,1),
     &          nfcon,fcon, fun,
     &          fvertx(1),dfrel,'!',istep)
c      write(*,*) 'Nstep =', istep

      return
      end ! simpop

