
      program tasopt
c==========================================================================
c
c     Transport Aircraft System OPTimization
c
c     Program for sizing and/or optimizing of transport aircraft.
c     Program modes and functions are listed below.
c
c     The program is structured as up to four nested loops:
c
c     "i" loop:  i = 1:ni
c      Loops over "i" parameter, typically in relatively coarse steps
c
c       "j" loop:  j = 1:nj
c       Loops over "j" parameter, typically in relatively fine steps
c
c         Optimization loop:  istep = 1:istepmax  (or less)
c         Iterates on selected optimization variables to minimize obj. function
c 
c           Weight/sizing loop:  iterw = 1:iterwmax
c           Iterates on aircraft weight and associated aircraft parameters
c
c     Usage:
c
c       % tasopt xxx  [ yyy ]
c
c     where "xxx" is an arbitrary case prefix, 
c     and "yyy" is an optional savefile prefix (could be same as xxx).
c
c
c     Actions:
c
c     1)  Reads and processes input file  xxx.tas 
c
c     2)  Writes output to file(s)
c              xxx.out           if no i,j looping is specified
c          or  xxx_iNN.out       if only i looping is specified
c          or  xxx_jMM.out       if only j looping is specified
c          or  xxx_iNN_jMM.out   if both i and j looping is specified
c       where "NN" and "MM" are the specific i,j values.
c       File outputs can be suppressed by setting flags in xxx.tas .
c
c     3) Writes plot-data output to files 
c          xxx_1.plt   contains Matlab plot titles
c          xxx_2.plt   contains Matlab plot data
c          xxx_3.plt   contains Matlab airplane stick figure data
c          xxx_g.plt   contains Gnuplot airplane stick figure data
c
c    The plot-data files are intended for plotting using the Matlab scripts  
c       apl1.m
c       apl2.m
c       apl3.m
c       apl4.m
c
c--------------------------------------------------------------------------
c 
c    Copyright   Mark Drela   2010
c    MIT Aero & Astro
c
c==========================================================================
 
      implicit real (a-h,l-z)

      include 'index.inc'
      include 'airf.inc'
      include 'arrs.inc'
      include 'fbl.inc'
      include 'trp.inc'
      include 'constants.inc'

      include 'time.inc'

      integer ni, nj, njd, nvar
      integer lu, lusav
      integer lu1,lu2,lu3,lu4,lu5,lu6,lu7,lu8,lu9,
     &        lu10,lu11,lu12,lu13,lu91,lu92

c---- max number of "i" and "j" parameters
      integer idim,jdim
      parameter (idim=10,
     &           jdim=51)
      parameter (idim1=idim+1,
     &           jdim1=jdim+1 )

      real vars(iototal,iototal+1,idim,jdim), dvarso(iototal)
      real dvmax(iototal)
      real fvertx(iototal+1, idim,jdim)

c---- "i" and "j" parameter values
      real parsi(idim1), parsj(jdim1)

c---- constraint function values
      integer ncdim
      parameter (ncdim=21)
      real confun(ncdim)

      integer nfn, iline
      character*128 fname, line

      logical Lconv, Lopconv, Lerr, Lsaveuse, Lheader, Lengoper

c---- engine off-design operating parameters
      integer neAdim, neAlti,
     &        neMdim, neMach,
     &        neFdim, neFset
      parameter (neAdim = 21,
     &           neMdim = 21,
     &           neFdim = 21 )
      real eAlti(neAdim),
     &     eMach(neMdim), 
     &     eFset(neFdim)

c---- TASOPT version number
      version = 2.16

cc---- zero out CPU time accumulators
c      t_all = 0.0
c      t_fusebl = 0.
c      t_surfdx = 0.
c      t_mcool = 0.
c      t_fusew = 0.
c      t_surfcm = 0.
c      t_surfw = 0.
c      t_htsize = 0.
c      t_balance = 0.
c      t_tfsize = 0.
c      t_tfoper = 0.
c      t_tfweight = 0.
c      t_surfcd = 0.
c      t_surfcd2 = 0.
c      t_cditrp = 0.
c      call tset(time0)

c---- expansion factor for initial simplex taken from the previous point 
c-    (optimum-point simplex will be nearly collaped, so it must be expanded)
      vexpi = 5.0

c---- dump separate engine file?
c-    (separately from usual .out file)
      Ltfwrite = .false.
c     Ltfwrite = .true.

c---- dump separate overall-profile file?
c-    (separately from usual .out file)?
      Lpfwrite = .false.
c     Lpfwrite = .true.

c---- fortran logical-unit i/o device numbers
      lu1  = 1     ! string    plot output file for Matlab
      lu2  = 2     ! parameter plot output file for Matlab
      lu3  = 3     ! geometry  plot output file for Matlab
      lu4  = 4     ! geometry  plot output file for gnuplot
      lu6  = 6     ! terminal output
      lu7  = 7     ! main input  file xxx.tas, optional input file xxx.tase
      lu8  = 8     ! save i/o    file yyy.sav
      lu9  = 9     ! main output file xxx.out
      lu10 = 10    ! ASWING output file xxx.asw
      lu11 = 11    ! Trefftz Plane output
      lu12 = 12    ! xxx.tase
      lu13 = 13    ! engine operating point output file xxx.oute
      lu91 = 91    ! trajectory on fan map
      lu92 = 92    ! mission profile summary

      bignum = 2.0 ** 1023
      do ig = 1, igtotal
        parg(ig) = bignum
      enddo
      do km = 1, nmisx
        do im = 1, imtotal
          parm(im,km) = bignum
        enddo
        do ip = 1, iptotal
          do ia = 1, iatotal
            para(ia,ip,km) = bignum
          enddo
          do ie = 1, ietotal
            pare(ie,ip,km) = bignum
          enddo
        enddo
      enddo

c---------------------------------------------
c---- fill parameters in constants.inc block
      pi = 3.1415926535897932384626
      gee = 9.81
      cpSL  = 1004.0
      gamSL = 1.4
      altkmSL = 0.0
      call atmos(altkmSL,TSL,pSL,rhoSL,aSL,muSL)
      RSL = pSL/(rhoSL*TSL)

      Tref = TSL
      pref = pSL

c---------------------------------------------
c---- unit conversion factors
      lb_N = 1.0 / 4.44822
      ft_m  = 1.0 / 0.3048
      kft_m = 1.0 / 0.3048 / 1000.0
      in_m = 39.37
      psi_Pa = 0.000145038
      nmi_m = 0.000539975
      hr_s = 1.0/3600.0
      lb_kg = 2.20462
      hp_W  = 1.0 / 745.7
      R_K = 1.8

c---------------------------------------------
c---- get case argument
      call getarg(1,arg1)
      narg1 = index(arg1,' ') - 1

      if(arg1 .eq. ' ') then
       stop 'Input file not specified'
      endif

c---- open and read input file
      fname = arg1(1:narg1) // '.tas'
      nfn = narg1 + 4
      open(lu7,file=fname,status='old',err=700)
      call getparm(lu7,idim,jdim,
     &                ni,nj,ispars,jspars,
     &                parsi,parsj,dvarso)
      close(lu7)

c---- try to open and read engine operating point matrix file
      fname = arg1(1:narg1) // '.tase'
      nfn = narg1 + 5

      Lengoper = .false.
      open(lu12,file=fname,status='old',err=15)
      iline = 0

      call getcval(lu12,iline,line)
      neAlti = neAdim
      call getrkey(line,' ', eAlti, neAlti)

      call getcval(lu12,iline,line)
      neMach = neMdim
      call getrkey(line,' ', eMach, neMach)

      call getcval(lu12,iline,line)
      neFset = neFdim
      call getrkey(line,' ', eFset, neFset)

      close(lu12)

      Lengoper = neAlti .gt. 0 
     &    .and.  neMach .gt. 0
     &    .and.  neFset .gt. 0

 15   continue

c---- first mission defines maximum range, payload, etc. for sizing airframe
      km = 1
      parg(igRange) = parm(imRange,km)
      parg(igWpay ) = parm(imWpay ,km)


c---- limit ni,nj if necessary
      if(ni .gt. idim) then
       write(*,3010) cpars(ispars), 'idim', idim
       ni = idim
      endif
      if(nj .gt. jdim) then
       write(*,3010) cpars(jspars), 'jdim', jdim
       nj = jdim
      endif
 3010 format(/' Limiting number of ',a,
     &          'values to array limit  ',a,' =', i4)

c---- set active-variable indices
      iv = 0
      do io = 1, iototal
        if(dvarso(io) .ne. 0.0) then
         iv = iv + 1
         iovar(iv) = io
        endif
      enddo

c---- number of optimization variables
      nvar = iv

c---- j increment for generating stick-figure geometry output
      jdel = (nj-1)/4 + 1

c---- initialize weights on first pass
      initwgt = 0

c---- initialize engine parameters on first pass
      initeng = 0

c---------------------------------------------
c---- open and read save file, if specified
      Lsaveuse = .false.
      narg2 = 0

      call getarg(2,arg2)
      if(arg2 .ne. ' ') then
        narg2 = index(arg2,' ') - 1
        fname = arg2(1:narg2) // '.sav'
        open(lu8,file=fname,status='old',err=50)
        call getsave(lu8,iototal,idim,jdim, 
     &              nvar,ni,nj, iovar,
     &              ispars,jspars,
     &              vars,Lerr )
        close(lu8)

        nfn = index(fname,' ')
        if(Lerr) then
         write(*,*)
         write(*,*) '** Cannot use optimization save file  ',
     &               fname(1:nfn)
        else
         Lsaveuse = .true.
         write(*,*)
         write(*,*) 'Will restart from optimization save file  ',
     &               fname(1:nfn)
        endif
        go to 55

 50     continue
        write(*,*)
        write(*,*) '** Save file  ', 
     &              fname(1:nfn),'  not found'
 55     continue
        write(*,*)
      endif

      
      if(Lsaveuse) then
c----- expand all saved simplexes
       do iv = 1, nvar
         dvmax(iv) = 0.
       enddo

c----- find max delta for each variable, among all saved simplexes
       do i = 1, ni
         do j = 1, nj
           do jv = 2, nvar+1
             do iv = 1, nvar
               delvar = vars(iv,jv,i,j) - vars(iv,1,i,j)
               dvmax(iv) = max( dvmax(iv) , abs(delvar) )
             enddo
           enddo
         enddo
       enddo

c----- find max relative delta among all variables, 
c-      compared to prescribed initial delta for each variable
       drmax = 0.
       do iv = 1, nvar
         io = iovar(iv)
c         write(*,'(1x,i4,6g13.5)') 
c     &    io, dvmax(iv), dvarso(io), dvmax(iv)/dvarso(io)
         drat = dvmax(iv)/dvarso(io)
         drmax = max( drmax , drat )
       enddo

c----- expand simplexes so they are about half as big
c-      as the prescribed initial sizes
       vexp = 0.5/drmax

       do i = 1, ni
         do j = 1, nj
           do jv = 2, nvar+1
             do iv = 1, nvar
               delvar = vars(iv,jv,i,j) - vars(iv,1,i,j)
               vars(iv,jv,i,j) = vars(iv,1,i,j) + vexp*delvar
             enddo
           enddo
         enddo
       enddo

      endif

c==========================================================================
c---- fuselage BL calculations
c---- assume this dissipation (normalized by rhoinf Vinf^3) is the same
c-     for all operating points 
c      do km = 1, nmission
c        ip = ipcruise1
c        call fusebl(pari,parg,para(1,ip,km))
c        Difsurf = para(iaDifsurf,ip,km)
c        Difwake = para(iaDifwake,ip,km)
c        Pinff = para(iaPinff,ip,km)
cc        Difuse = Difsurf + Difwake
cc        write(*,*) km, para(iafexcdf,ip,km), Difuse
c        do ip = 1, iptotal
c          para(iaDifsurf,ip,km) = Difsurf
c          para(iaDifwake,ip,km) = Difwake
c          para(iaPinff,ip,km) = Pinff
c        enddo
c      enddo

      if(Lopt .and. Lsavwrite) then
c----- write out save file header
c       if(narg2.eq.0) then
c        fname = arg1(1:narg1) // '.sav'
c       else
c        fname = arg2(1:narg2) // '.sav'
c       endif
       lusav = 8
       fname = arg1(1:narg1) // '.sav'
       open(lusav,file=fname,status='unknown')
       nfn = index(fname,' ')
       write(*,*) 'Will write to save file:  ', fname(1:nfn)
       write(*,*)
       call wrtsave0(lusav,iototal,idim,jdim, 
     &               nvar,iovar,
     &               ispars,jspars )
       close(lusav)
      endif

      fnameg = arg1(1:narg1) // '_g.plt'

C=========================================================================
c---- go over i,j points
      do 200 i = 1, ni
      do 100 j = 1, nj

c------------------------------------------------------------------------
c---- set prescribed sequenced parameters for this i,j point

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isRange) Range = parsi(i)
      if(jspars.eq.isRange) Range = parsj(j)
      if(ispars.eq.isRange .or.
     &   jspars.eq.isRange      ) then
        parg(igRange) = Range
        km = 1
        parm(imRange,km) = Range
c        do km = 1, nmission
c          parm(imRange,km) = Range
c        enddo
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isMach) Mach = parsi(i)
      if(jspars.eq.isMach) Mach = parsj(j)
      if(ispars.eq.isMach .or.
     &   jspars.eq.isMach      ) then
        do km = 1, nmission
          do ip = ipclimbn, ipdescent1
            para(iaMach,ip,km) = Mach
          enddo
        enddo
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isNmax  ) parg(igNlift) = parsi(i)
      if(jspars.eq.isNmax  ) parg(igNlift) = parsj(j)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.issigfac) parg(igsigfac) = parsi(i)
      if(jspars.eq.issigfac) parg(igsigfac) = parsj(j)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isCL ) CL = parsi(i)
      if(jspars.eq.isCL ) CL = parsj(j)
      if(ispars.eq.isCL .or.
     &   jspars.eq.isCL ) then
        do km = 1, nmission
          do ip = ipcruise1, ipcruisen
            para(iaCL,ip,km) = CL
          enddo
        enddo
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isAR    ) parg(igAR) = parsi(i)
      if(jspars.eq.isAR    ) parg(igAR) = parsj(j)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.issweep ) then
       parg(igsweep ) = parsi(i)
cc     parg(igsweeph) = parsi(i)
      endif
      if(jspars.eq.issweep ) then
       parg(igsweep)  = parsj(j)
cc     parg(igsweeph) = parsj(j)
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isetas  ) parg(igetas) = parsi(i)
      if(jspars.eq.isetas  ) parg(igetas) = parsj(j)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isT4CR  ) Tt4cr = parsi(i)
      if(jspars.eq.isT4CR  ) Tt4cr = parsj(j)
      if(ispars.eq.isT4CR .or.
     &   jspars.eq.isT4CR  ) then
        do km = 1, nmission
          do ip = ipcruise1, ipcruisen
            pare(ieTt4,ip,km) = Tt4cr
          enddo
        enddo
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isT4TO  ) Tt4TO = parsi(i)
      if(jspars.eq.isT4TO  ) Tt4TO = parsj(j)
      if(ispars.eq.isT4TO .or.
     &   jspars.eq.isT4TO  ) then
        do km = 1, nmission
          pare(ieTt4,ipstatic ,km) = Tt4TO
          pare(ieTt4,iprotate ,km) = Tt4TO
          pare(ieTt4,iptakeoff,km) = Tt4TO
        enddo
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isTmetal) Tmetal = parsi(i)
      if(jspars.eq.isTmetal) Tmetal = parsj(j)
      if(ispars.eq.isTmetal .or.
     &   jspars.eq.isTmetal ) then
        parg(igTmetal) = Tmetal
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isOPR   ) OPR = parsi(i)
      if(jspars.eq.isOPR   ) OPR = parsj(j)
      if(ispars.eq.isOPR  .or.
     &   jspars.eq.isOPR   ) then
        ip = ipcruise1
        km = 1
        pihc = OPR/pare(iepilc,ip,km)
        do km = 1, nmission
          do ip = 1, iptotal
cc            pihc = OPR/pare(iepilc,ip,km)
c            write(*,'(1x,a,2x,3f8.3)') 
c     &        cplab(ip),  opr, pihc, pare(iepilc,ip,km)
            pare(iepihc,ip,km) = pihc
          enddo
        enddo
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isFPR   ) pif = parsi(i)
      if(jspars.eq.isFPR   ) pif = parsj(j)
      if(ispars.eq.isFPR  .or.
     &   jspars.eq.isFPR   ) then
        do km = 1, nmission
          do ip = 1, iptotal
            pare(iepif,ip,km) = pif
          enddo
        enddo
      endif

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.islBFmax) parg(iglBFmax) = parsi(i)
      if(jspars.eq.islBFmax) parg(iglBFmax) = parsj(j)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isbmax) parg(igbmax) = parsi(i)
      if(jspars.eq.isbmax) parg(igbmax) = parsj(j)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
      if(ispars.eq.isalt) alt = parsi(i)
      if(jspars.eq.isalt) alt = parsj(j)
      if(ispars.eq.isalt .or.
     &   jspars.eq.isalt      ) then
        do km = 1, nmission
          para(iaalt,ipcruise1,km) = alt
        enddo
      endif

c----------------------------------------------------------------------
c---- process this parameter point i,j

c---- print parameter i,j values to be helpful
      if(ispars.ne.0 .and. jspars.ne.0) then
        write(*,3300) i, cpars(ispars), parsi(i), 
     &                j, cpars(jspars), parsj(j)
      elseif(ispars.ne.0) then
        write(*,3200) i, cpars(ispars), parsi(i)
      elseif(jspars.ne.0) then
        write(*,3100) j, cpars(jspars), parsj(j)
      endif
 3300 format( ' i =', i2, ': ', a7, '=', g13.5,
     &     6x,' j =', i2, ': ', a7, '=', g13.5 )
 3200 format( ' i =', i2, ': ', a7, '=', g13.5 )
 3100 format( ' j =', i2, ': ', a7, '=', g13.5 )

c=============================================================================

      kmdes = 1
      if(.not. Lopt) then
c------ no optimization... just size aircraft directly for the first mission
        iairf = 1
        call wsize(pari,
     &             parg,
     &             parm(1,kmdes),
     &             para(1,1,kmdes),
     &             pare(1,1,kmdes),
     &             iterwmax,wrlx1,wrlx2,wrlx3,
     &             initwgt,initeng,iairf,
     &             ichoke5(1,kmdes),ichoke7(1,kmdes),
     &             Litprint,Lconv)

c------ calculate final Fobj value
c        nvopt = 0
c        istep = 0
c        call fobj(nvopt,vopt,
c     &            ncon,confun, fun,func, dfrel,'-',istep)

c------ calculate fuel-burn for off-design missions
        iairf = 2   !###

        do km = 2, nmission
cc-------- initialize all off-design variables to design-mission variables
c          BUG:  This im loop should not be here   22 Dec 16
c          do im = 1, imtotal
c            parm(im,km) = parm(im,kmdes)
c          enddo
          do ip = 1, iptotal
            do ia = 1, iatotal
              para(ia,ip,km) = para(ia,ip,kmdes)
            enddo
            do ie = 1, ietotal
              pare(ie,ip,km) = pare(ie,ip,kmdes)
            enddo
          enddo

c-------- tell subsequent routines that engine state is initialized
          initeng = 1 

c-------- converge off-design mission
          call woper(pari,
     &               parg,
     &               parm(1,km),
     &               para(1,1,km),
     &               pare(1,1,km),
     &               para(1,1,kmdes),
     &               pare(1,1,kmdes),
     &               iterfmax,initeng,iairf,
     &               ichoke5(1,kmdes),ichoke7(1,kmdes),
     &               Litprint,Lconv)
        enddo

c------ calculate noise
        do km = 1, nmission
          if(km.eq.1) then   !###
            iairf = 1 
          else
            iairf = 2   !###
          endif
          call noise(pari,parg,parm(1,km),para(1,1,km),pare(1,1,km),
     &               initeng,iairf,ichoke5(1,km),ichoke7(1,km))
        enddo

c------ output point i,j
        if(Loutwrite) then
         call outwrt(lu9,lu91,lu92,i,j,ispars,jspars)
        endif


c        arg1 = 'tmp'
c        narg1 = 3
c        initwgt = 1
c        initeng = 1
c        iairf = 1
c        iterwmax = 3
c        call wsize(pari,
c     &             parg,
c     &             parm(1,kmdes),
c     &             para(1,1,kmdes),
c     &             pare(1,1,kmdes),
c     &             iterwmax,wrlx1,wrlx2,wrlx3,
c     &             initwgt,initeng,iairf,
c     &             ichoke5(1,kmdes),ichoke7(1,kmdes),
c     &             Litprint,Lconv)
c         call outwrt(lu9,lu91,lu92,i,j,ispars,jspars)


c=============================================================================
      else
c------ optimize selected variables using simplex method

        if(Lsaveuse) then
c-------- don't initialize anything... 
c-         just use current vars(....) read in from save file

        else
c-------- no saved variables... initialize them
c
c--------------------------------------------------------
          if(i.eq.1 .and. j.eq.1) then
c---------- first point in parameter grid... set initial optimization variables
            jv = 1
            k = 1
            do iv = 1, nvar
             io = iovar(iv)
             if(io.eq.ioCL   ) vars(iv,jv,i,j)= para(iaCL,ipcruise1,k)
             if(io.eq.ioAR   ) vars(iv,jv,i,j)= parg(igAR) 
             if(io.eq.iosweep) vars(iv,jv,i,j)= parg(igsweep)
             if(io.eq.iohboxo) vars(iv,jv,i,j)= parg(ighboxo)
             if(io.eq.iohboxs) vars(iv,jv,i,j)= parg(ighboxs)
             if(io.eq.iolams ) vars(iv,jv,i,j)= parg(iglambdas)
             if(io.eq.iolamt ) vars(iv,jv,i,j)= parg(iglambdat)
             if(io.eq.iorcls ) vars(iv,jv,i,j)= para(iarcls,ipcruise1,k)
             if(io.eq.iorclt ) vars(iv,jv,i,j)= para(iarclt,ipcruise1,k)
             if(io.eq.ioFPR  ) vars(iv,jv,i,j)= pare(iepif,ipcruise1,k)
             if(io.eq.ioBPR  ) vars(iv,jv,i,j)= pare(ieBPR,ipcruise1,k)
             if(io.eq.ioalt  ) vars(iv,jv,i,j)= para(iaalt,ipcruise1,k)
             if(io.eq.ioT4CR ) vars(iv,jv,i,j)= pare(ieTt4,ipcruise1,k)
             if(io.eq.ioT4TO ) vars(iv,jv,i,j)= pare(ieTt4,iptakeoff,k)
             if(io.eq.ioOPR  ) vars(iv,jv,i,j)= pare(iepihc,ipcruise1,k)
     &                                        * pare(iepilc,ipcruise1,k)
            enddo

c---------- set remaining points in simplex using specified perturbations
            do jv = 2, nvar+1
              do iv = 1, nvar
                vars(iv,jv, i,j) = vars(iv,1, i,j)
              enddo
            enddo
            do iv = 1, nvar
              jv = iv + 1
              io = iovar(iv)
              vars(iv,jv, i,j) = vars(iv,jv, i,j) + dvarso(io)
            enddo

c--------------------------------------------------------
          elseif(j.eq.1) then
c---------- use adjacent previously-converged i-1,j variables for initial guess
ccc            do jv = 1, nvar+1
            do jv = 1, 1
              do iv = 1, nvar
                vars(iv,jv,i,j) = vars(iv,jv,i-1,j)
              enddo
            enddo

c---------- set remaining points in simplex using specified perturbations
            do jv = 2, nvar+1
              do iv = 1, nvar
                vars(iv,jv, i,j) = vars(iv,1, i,j)
              enddo
            enddo
            do iv = 1, nvar
              jv = iv + 1
              io = iovar(iv)
              vars(iv,jv, i,j) = vars(iv,jv, i,j) + dvarso(io)
            enddo

cc---------- expand simplex by factor of vexp so it can move faster initially
c            vexp = vexpi
c            do jv = 2, nvar+1
c              do iv = 1, nvar
c                 delvar = vars(iv,jv,i,j) - vars(iv,1,i,j)
c                 vars(iv,jv,i,j) = vars(iv,1,i,j) + vexp*delvar
c              enddo
c            enddo
c
c--------------------------------------------------------
          else
c---------- use adjacent previously-converged i,j-1 variables for initial guess
            do jv = 1, nvar+1
              do iv = 1, nvar
                vars(iv,jv,i,j) = vars(iv,jv,i,j-1)
              enddo
            enddo

c---------- set remaining points in simplex using specified perturbations
            do jv = 2, nvar+1
              do iv = 1, nvar
                vars(iv,jv, i,j) = vars(iv,1, i,j)
              enddo
            enddo
            do iv = 1, nvar
              jv = iv + 1
              io = iovar(iv)
              vars(iv,jv, i,j) = vars(iv,jv, i,j) + dvarso(io)
            enddo

cc---------- expand simplex by factor of vexp so it can move faster initially
c            vexp = vexpi
c            do jv = 2, nvar+1
c              do iv = 1, nvar
c                 delvar = vars(iv,jv,i,j) - vars(iv,1,i,j)
c                 vars(iv,jv,i,j) = vars(iv,1,i,j) + vexp*delvar
c              enddo
c            enddo

          endif
c--------------------------------------------------------
        endif

        ioutnum = (istepmax+istepout-1)/istepout
        istepdone = 0
        do iout = 1, ioutnum
c-------- perform up to isteplast opt. steps on the vars(...,i,j) simplex
          isteplast = min( istepdone+istepout , istepmax )
          call simpop(iototal,nvar, dvarso,
     &                vars(1,1,i,j),fvertx(1,i,j),
     &                Wftol,Wfdel,istepdone,isteplast,Lopconv)
ccc        call gradop(ndim,n, vopt,dvopt,fopt,
ccc     &                  ftol,fdel,nstepmax)
          istepdone = isteplast

c-------- optimization-convergence index,  < 1 if converged
          conval = abs(Wfdel) / Wftol

c-------- calculate off-design missions which have zero weight
c-        (these were skipped by simpop call above, for speed)
          do km = 2, nmission
            if(parm(imwOpt,km) .eq. 0.0) then
             iairf = 2   !###
             call woper(pari,parg,
     &               parm(1,km),
     &               para(1,1,km),
     &               pare(1,1,km),
     &               para(1,1,kmdes),
     &               pare(1,1,kmdes),
     &               iterfmax,initeng,iairf,
     &               ichoke5(1,kmdes),ichoke7(1,kmdes),
     &               Litprint,Lconv)
           endif
          enddo  ! next km  (mission index)

c-------- store updated optimization variables in jv=1 vector, converged or not
          jv = 1
          do km = 1, nmission
           do iv = 1, nvar
            io = iovar(iv)
            if(io.eq.ioCL   ) para(iaCL,ipcruise1,km) = vars(iv,jv,i,j)

            if(io.eq.ioAR   ) parg(igAR)      = vars(iv,jv,i,j)
            if(io.eq.iosweep) then
             parg(igsweep)   = vars(iv,jv,i,j)
             parg(igsweeph)  = vars(iv,jv,i,j)      ! ###
            endif

            if(io.eq.iohboxo) parg(ighboxo)   = vars(iv,jv,i,j)
            if(io.eq.iohboxs) parg(ighboxs)   = vars(iv,jv,i,j)
  
            if(io.eq.iolams ) parg(iglambdas) = vars(iv,jv,i,j)
            if(io.eq.iolamt ) parg(iglambdat) = vars(iv,jv,i,j)

            if(io.eq.iorcls ) then
             do ip = ipclimb1+1, ipcruisen-1
               para(iarcls,ip,km) = vars(iv,jv,i,j)
             enddo
            endif

            if(io.eq.iorclt ) then
             do ip = ipclimb1+1, ipcruisen-1
               para(iarclt,ip,km) = vars(iv,jv,i,j)
             enddo
            endif

            if(io.eq.ioFPR ) pare(iepif ,ipcruise1,km) = vars(iv,jv,i,j)
            if(io.eq.ioBPR ) pare(ieBPR ,ipcruise1,km) = vars(iv,jv,i,j)
            if(io.eq.ioalt ) para(iaalt ,ipcruise1,km) = vars(iv,jv,i,j)
            if(io.eq.ioT4CR) pare(ieTt4 ,ipcruise1,km) = vars(iv,jv,i,j)
            if(io.eq.ioT4TO) pare(ieTt4 ,iptakeoff,km) = vars(iv,jv,i,j)
            if(io.eq.ioOPR ) pare(iepilc,ipcruise1,km) = vars(iv,jv,i,j)
     &                      /pare(iepihc,ipcruise1,km)
           enddo
          enddo   ! next km  (mission index)

c-------- write converged simplex to save file
          if(Lsavwrite) then
           call wrtsave1(lusav,iototal,idim,jdim, 
     &                   nvar, i, j,
     &                   vars )
          endif

c-------- calculate noise
          do km = 1, nmission

          if(km.eq.1) then   !###
            iairf = 1 
          else
            iairf = 2   !###
          endif

            call noise(pari,parg,parm(1,km),para(1,1,km),pare(1,1,km),
     &                 initeng,iairf,ichoke5(1,km),ichoke7(1,km))
          enddo

c-------- output point i,j
          if(Loutwrite) then
           call outwrt(lu9,lu91,lu92,i,j,ispars,jspars)
          endif

c          iluf = 30 + 2*(i-1)
c          if(j.eq.1) then
c          write(iluf,'(a,a,a,a)')
c     &     '#Tmetal PFEI_rel  PFEI  WMTO_rel  WMTO  ',
c     &     '  dfan   Tt4_TO  Tt4_CR   fc     ',
c     &     ' FPR_D   BPR_D   CL_CR   L/D_CR  h_CR'
c          endif
c          write(iluf,8844) 
c     &     parg(igTmetal),
c     &     parg(igPFEI)/6.5204,
c     &     parg(igPFEI),
c     &     parg(igWMTO)*lb_N/162117.0,
c     &     parg(igWMTO)*lb_N,
c     &     parg(igdfan)*in_m,
c     &     pare(ieTt4,iptakeoff,1),
c     &     pare(ieTt4,ipcruise1,1),
c     &     pare(iefc,iptakeoff,1),
c     &     pare(iepif,ipcruise1,1),
c     &     pare(ieBPR,ipcruise1,1),
c     &     para(iaCL,ipcruise1,1),
c     &     para(iaalt,ipcruise1,1)*ft_m/1000.0
c 8844     format(1x, f5.0, 
c     &             f8.5, f9.4,
c     &             f8.5, f9.0,
c     &             f8.3,
c     &             2f8.1, f9.5, f8.4, f8.4, 
c     &             f8.4, f8.3, f8.3)


c-------- jump out is simpop converged
          if(Lopconv) go to 80
        enddo   ! next iout (block of istepout steps)

 80     continue
      endif

c=============================================================================

c----------------------------------------------------------------------
c---- append point i,j data to plot-output files

      if(i.eq.1 .and. j.eq.1) then
c----- open plot-output files only for first i,j point
       narg1 = index(arg1,' ') - 1
       fname = arg1(1:narg1) // '_1.plt'
       open(lu1,file=fname,status='unknown')
       fname = arg1(1:narg1) // '_2.plt'
       open(lu2,file=fname,status='unknown')
       fname = arg1(1:narg1) // '_3.plt'
       open(lu3,file=fname,status='unknown')
       Lheader = .true.
      else
       Lheader = .false.
      endif

      if(jspars.eq.0) then
c----- no j parameter specified... just write out zero
       parsjspec = 0.
      else
       parsjspec = parsj(j)
      endif

c---- write to plot-output files
      call pltwrt(lu1,lu2,lu3,Lheader,
     &            i,j,ni,nj,jdel,parsjspec)

c---- plot "stick figure" data file
cc    xorg = 0.0          ! leave x coordinates unchanged
      xorg = parg(igxNP)  ! shift x coordinates to put x=0 at neutral point
      call picwrt(lu4,fnameg,pari,parg,xorg)

ccc   call picidr(lu4,arg1,i,j,ispars,jspars,pari,parg,parg(igxNP))

c----------------------------------------------------------------------
c---- output Trefftz Plane quantities at a few selected mission points
      if(Ltrpwrite) then
        km = 1
        do ip = 1, iptotal
          if(ip.eq.iptakeoff  .or.
     &       ip.eq.ipcruise1  .or.
     &       ip.eq.ipdescentn      ) then
            fname = arg1(1:narg1) // '_tp' // cplab(ip) // '.dat'

            call cditrp(pari,parg,para(1,ip,km))
            open(lu11,file=fname,status='unknown')
            call trpwrt(lu11, nsurf,ifrst,ilast, parg(igb),
     &                  yc, zc, gc, vc, wc, vnc, ycp, zcp )
            close(lu11)
          endif
        enddo
      endif
c----------------------------------------------------------------------

 100  continue  ! with next j
 200  continue  ! with next i
c=========================================================================

      close(lu1)
      close(lu2)
      close(lu3)
      if(Lopt .and. Lsavwrite) close(lusav)

c      if(Lopt .and. Lsavwrite) then
cc----- write out save file
c       if(narg2.eq.0) then
c        fname = arg1(1:narg1) // '.sav'
c       else
c        fname = arg2(1:narg2) // '.sav'
c       endif
c       open(lu8,file=fname,status='unknown')
c       nfn = index(fname,' ')
c       write(*,*) 'Writing  save  file:  ', fname(1:nfn)
c       write(*,*)
c       call wrtsave(lu8,iototal,idim,jdim, 
c     &              nvar,ni,nj, iovar,
c     &              ispars,jspars,
c     &              vars )
c      endif

c-----------------------------------------------------
      if(Laswwrite) then
       fname = arg1(1:narg1) // '.asw'
       nfn = narg1 + 4
       ip = ipcruise1
       km = 1
       open(lu10,file=fname,status='unknown')
       write(*,*) 'Writing ASWING file:  ', fname(1:nfn)
       call aswout(lu10,pari,parg,para(1,ip,km),configname)
       close(lu10)
      endif


      if(Lengoper) then
c----- compute and output engine operating points
       fname = arg1(1:narg1) // '.oute'
       nfn = narg1 + 5
       open(lu13,file=fname,status='unknown')
       write(*,*) 'Writing engine operating points file:  ',fname(1:nfn)
       call eopwrt(lu13,neAlti,neMach,neFset, eAlti,eMach,eFset)
       close(lu13)
      endif

c=========================================================================
c---- total elapsed computation time
c      call tadd(time0,t_all)
cc
c---- sum of all the individual routine CPU times
c      t_sum = 
c     &  t_fusebl +
c     &  t_surfdx +
c     &  t_mcool +
c     &  t_fusew +
c     &  t_surfcm +
c     &  t_surfw +
c     &  t_htsize +
c     &  t_balance +
c     &  t_tfsize +
c     &  t_tfoper +
c     &  t_tfweight +
c     &  t_surfcd +
c     &  t_surfcd2 +
c     &  t_cditrp
c
c      write(*,*) 
c      write(*,*) 'CPU usage...'
c      write(*,1122) 'fusebl  :', t_fusebl  , t_fusebl  /t_sum*100.0
c      write(*,1122) 'surfdx  :', t_surfdx  , t_surfdx  /t_sum*100.0
c      write(*,1122) 'mcool   :', t_mcool   , t_mcool   /t_sum*100.0
c      write(*,1122) 'fusew   :', t_fusew   , t_fusew   /t_sum*100.0
c      write(*,1122) 'surfcm  :', t_surfcm  , t_surfcm  /t_sum*100.0
c      write(*,1122) 'surfw   :', t_surfw   , t_surfw   /t_sum*100.0
c      write(*,1122) 'htsize  :', t_htsize  , t_htsize  /t_sum*100.0
c      write(*,1122) 'balance :', t_balance , t_balance /t_sum*100.0
c      write(*,1122) 'tfsize  :', t_tfsize  , t_tfsize  /t_sum*100.0
c      write(*,1122) 'tfoper  :', t_tfoper  , t_tfoper  /t_sum*100.0
c      write(*,1122) 'tfweight:', t_tfweight, t_tfweight/t_sum*100.0
c      write(*,1122) 'surfcd  :', t_surfcd  , t_surfcd  /t_sum*100.0
c      write(*,1122) 'surfcd2 :', t_surfcd2 , t_surfcd2 /t_sum*100.0
c      write(*,1122) 'cditrp  :', t_cditrp  , t_cditrp  /t_sum*100.0
c      write(*,1122) 'Sum     :', t_sum     , t_sum     /t_sum*100.0
c      write(*,1122) 'ALL     :', t_all     
c      write(*,*)
c 1122 format(1x,a,f13.7,' s', f13.5,' %')

      stop

c-----------------------------------------------------
 700  continue
      write(*,*) 'OPEN error on file ', fname(1:nfn)
      stop

      end ! tasopt



      subroutine outwrt(lu,lu1,lu2,i,j,ispars,jspars)
      implicit real (a-h,l-z)

      include 'index.inc'
      include 'airf.inc'
      include 'arrs.inc'
      include 'fbl.inc'
      include 'trp.inc'
      include 'constants.inc'

      integer lu, lu1, lu2

      integer ni, nj, njd, nvar
      integer naf, nfn

      parameter(ixydim=15)
      integer nw,nh,nf,ne
      real xw(ixydim), yw(ixydim),
     &     xh(ixydim), yh(ixydim),
     &     xf(ixydim), yf(ixydim),
     &     xe(ixydim), ye(ixydim)

      character*3 cmach, ci, cj
      character*128 fname
      character*20 cpname

      km = 1
      ip = ipcruise1
      
      LoD = para(iaCL,ip,km)
     &    / para(iaCD,ip,km)
      cosL = cos(parg(igsweep) * pi/180.0)
      Mperp = para(iaMach,ip,km)*cosL
      CLperp = para(iaCL,ip,km)/cosL**2


      izero = ichar('0')

      Mach = para(iaMach,ipcruise1,km)
      idig1 = int(Mach*10.0)
      idig2 = int((Mach-0.1*float(idig1))*100.0+0.01)
      idig3 = int((Mach-0.1*float(idig1)-0.01*float(idig2))*1000.0+0.01)
      cmach = char(izero+idig1)
     &     // char(izero+idig2)
     &     // char(izero+idig3)

      idig1 = i / 10
      idig2 = i - idig1*10
      ci = 'i' // char(izero+idig1) // char(izero+idig2)

      idig1 = j / 10
      idig2 = j - idig1*10
      cj = 'j' // char(izero+idig1) // char(izero+idig2)

c      if(bs.gt.bo) then
cc----- strut stuff
c       Tstrutp = Astrut*sigstrut
c       ssp = 0.5*(bs-bo)/cosL
c       lsp = sqrt(zs**2 + ssp**2)
c       Pstrutp = Tstrutp * ssp/lsp
c       cop = co*cosL
c       csp = cs*cosL
c       EIo = Ecap*cop**4 * (wbox/12.0)*(hbox**3 - (hbox-2.0*tbcapo)**3)
c       EIs = Ecap*csp**4 * (wbox/12.0)*(hbox**3 - (hbox-2.0*tbcaps)**3)
c       EI = 0.5*(EIo+EIs)
c       Pcrit = pi**2 * EI / ssp**2
c      else
c       Pstrutp = 0.
c       Pcrit   = 1.0
c      endif

c----- construct output filename
ccc    fname = arg1(1:narg1) // '_parm_' // cmach // '.out'
       if(ispars .ne. 0 .and. jspars .ne. 0) then
        fname = arg1(1:narg1) // '_' // ci // '_' // cj // '.out'
       elseif(ispars .ne. 0) then
        fname = arg1(1:narg1) // '_' // ci // '.out'
       elseif(jspars .ne. 0) then
        fname = arg1(1:narg1) // '_' // cj // '.out'
       else
        fname = arg1(1:narg1) // '.out'
       endif
       open(lu,file=fname,status='unknown')
        nfn = index(fname,' ')
       write(*,*)
       write(*,*) 'Writing output file:  ', fname(1:nfn)

 1005 format(1x,a,a,a)
 1010 format(
     &/' =============================================================')
 1015 format(1x,'TASOPT v', f5.2)
 1020 format(
     &/' -----------------------------------------------------------')
 1040 format(
     &/' - - - - - - - - - - - - - - - - - - - - - - - - - - - - -')

       write(lu,1010)
       write(lu,1015) version
       write(lu,1020)
       write(lu,1005) 'Config:  ', configname
       write(lu,*)
       write(lu,1005) 'Case:  ', casename1
       write(lu,1005) '    :  ', casename2
       write(lu,*)

       do iairf = 1, nairf
         naf = index(airfile(iairf),' ')
         write(lu,1005) 'Airfoil database used:  ',airfile(iairf)(1:naf)
       enddo

       Optsum = 0.
       do km = 1, nmission
         Optsum = Optsum + parm(imwOpt,km)*parm(imPFEI,km)
       enddo
       parg(igPFEI) = Optsum

       write(lu,1020)
       write(lu,'(a,f8.4,a)') ' Fleet PFEI =',parg(igPFEI),' KJ/kg-km'

       write(lu,1020)
       write(lu,*) 'Airframe parameters...'
       write(lu,*)
       call geowrt(lu,pari,parg)

       if(Lfblwrite) then
        write(lu,1020)
        write(lu,*) 'Fuselage BL+Wake development...'
        write(lu,*)
        ip = ipcruise1
        km = 1
        Mach   = para(iaMach,ip,km)
        Reunit = para(iaReunit,ip,km)
        call blfwrt(lu,nbl,iblte,Mach,Reunit,
     &              xbl,zbl,sbl,bbl,
     &              uinv,uebl,
     &              dsbl,thbl,tsbl,dcbl,
     &              ctbl,cfbl,cdbl,hkbl,phbl )
c        lu2 = 11
c        call blfwrt2(lu2,nbl,iblte,Mach,Reunit,
c     &              xbl,zbl,sbl,bbl,
c     &              uinv,uebl,
c     &              dsbl,thbl,tsbl,dcbl,
c     &              ctbl,cfbl,cdbl,hkbl,phbl )
       endif

c=================================================================
c----- write outputs for all missions
      do 500 km = 1, nmission
     
       write(lu,1010)
       write(lu,'(a,i4)') ' Fleet mission', km
       write(lu,*)
       write(lu,'(a,f8.5)') ' wOpt =', parm(imwOpt,km)

       ip = ipcruise1
       Mach = para(iaMach,ip,km)
       altkm = para(iaalt,ip,km)/1000.0
       call atmos(altkm, T0,p0,rho0,a0,mu0)
       Reunit = Mach*a0 * rho0/mu0

       write(lu,1020)
       ip1 = ipcruise1
       ipn = ipcruisen

       Wfburn = parm(imWfuel,km)/(1.0+parg(igfreserve))
       ffuelb = Wfburn/parm(imWTO,km)
       LoD = para(iaCL,ip1,km)/para(iaCD,ip1,km)
       write(lu,*) 'Cruise performance...'
       write(lu,*)
       write(lu,'(a,f8.0,a)') ' Range =',parm(imRange,km)*nmi_m,' nmi'
       write(lu,'(a,f8.4,a)') ' M_CR  =',para(iaMach,ip1,km)
       write(lu,'(a,f8.3,a)') ' h_CR1 =',para(iaalt,ip1,km)*kft_m,' kft'
       write(lu,'(a,f8.3,a)') ' h_CR2 =',para(iaalt,ipn,km)*kft_m,' kft'
       write(lu,'(a,f9.5,a)') ' Wf/W  =',ffuelb
       write(lu,'(a,f8.3,a)') ' L/D   =',LoD
       write(lu,'(a,f8.4,a)') ' PFEI  =',parm(imPFEI,km), ' KJ/kg-km'

       write(lu,1020)
       write(lu,*) 'Takeoff performance...'
       write(lu,*)
       call tofwrt(lu,parg,parm(1,km),para(1,1,km),pare(1,1,km))

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
       write(lu,1020)
       write(lu,*) 'Mission profile summary...'
       write(lu,*)
       call prfwrt(lu,parg,parm(1,km),para(1,1,km),pare(1,1,km))

C- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
       write(lu,1020)
       write(lu,*) 'Aero, Engine parameters...'
       write(lu,1040)

 2010  format(1x,a,':',2x,a)

       write(lu,2010) cplab(iprotate), 'Rotation...'
       call airwrt(lu,cplab(iprotate),para(1,iprotate,km),parg)
       call engwrt(lu,cplab(iprotate),pare(1,iprotate,km))
       write(lu,1040)

       write(lu,2010) cplab(iptakeoff), 'Takeoff...'
       call airwrt(lu,cplab(iptakeoff),para(1,iptakeoff,km),parg)
       call engwrt(lu,cplab(iptakeoff),pare(1,iptakeoff,km))
       write(lu,1040)

       write(lu,2010) cplab(ipcutback), 'Cutback...'
       call airwrt(lu,cplab(ipcutback),para(1,ipcutback,km),parg)
       call engwrt(lu,cplab(ipcutback),pare(1,ipcutback,km))
       write(lu,1040)

       ipstep = 1
c      ipstep = ipclimbn - ipclimb1 - 1
       do ip = ipclimb1, ipclimbn, ipstep
         inum = ichar('0') + ip - ipclimb1 + 1
         cpname = 'Climb' // char(inum) // '...'
         write(lu,2010) cplab(ip), cpname
         call airwrt(lu,cplab(ip),para(1,ip,km),parg)
         call engwrt(lu,cplab(ip),pare(1,ip,km))
         write(lu,1040)
       enddo

       ipstep = 1
c      ipstep = ipcruisen - ipcruise1 - 1
       do ip = ipcruise1, ipcruisen, ipstep
         inum = ichar('0') + ip - ipcruise1 + 1
         cpname = 'Cruise' // char(inum) // '...'
         write(lu,2010) cplab(ip), cpname
         call airwrt(lu,cplab(ip),para(1,ip,km),parg)
         call engwrt(lu,cplab(ip),pare(1,ip,km))
         write(lu,1040)
       enddo

       ipstep = 1
c      ipstep = ipdescentn - ipdescent1 - 1
       do ip = ipdescent1, ipdescentn, ipstep
         inum = ichar('0') + ip - ipdescent1 + 1
         cpname = 'Descent' // char(inum) // '...'
         write(lu,2010) cplab(ip), cpname
         call airwrt(lu,cplab(ip),para(1,ip,km),parg)
         call engwrt(lu,cplab(ip),pare(1,ip,km))
         write(lu,1040)
       enddo

 500  continue ! next mission

      close(lu)


      if(Ltfwrite) then
       fname = 'tfan_' // cmach // '.dat'
       open(lu1,file=fname,status='unknown')
       km = 1

c       ip = ipcruise1
c       write(lu1,'(a,3f10.5)') '#', 
c     &   pare(iepif,ip,km), pare(iepilc,ip,km), pare(iepihc,ip,km)

       call mapwrt(lu1,pare(1,ipstatic ,km),cplab(ipstatic ),1)
       call mapwrt(lu1,pare(1,iprotate ,km),cplab(iprotate ),0)
       call mapwrt(lu1,pare(1,iptakeoff,km),cplab(iptakeoff),0)
       call mapwrt(lu1,pare(1,ipcutback,km),cplab(ipcutback),0)
       write(lu1,*)
       do ip = ipclimb1, ipclimbn
         call mapwrt(lu1,pare(1,ip,km),cplab(ip),0)
       enddo
       write(lu1,*)
       do ip = ipcruise1, ipcruisen
         call mapwrt(lu1,pare(1,ip,km),cplab(ip),0)
       enddo
       write(lu1,*)
       do ip = ipdescent1, ipdescentn
         call mapwrt(lu1,pare(1,ip,km),cplab(ip),0)
       enddo
       close(lu1)
      endif

      if(Lpfwrite) then
       fname = 'prof_' // cmach // '.dat'
       open(lu2,file=fname,status='unknown')
       km = 1
       call prfwrt(lu2,parg,parm(1,km),para(1,1,km),pare(1,1,km))
       close(lu2)
      endif

c      write(*,*)
c      write(*,1020)
c      call prfwrt(6,parg,para,pare)

      return
      end ! outwrt



      subroutine eopwrt(lu, neAlti,neMach,neFset, eAlti,eMach,eFset)
      implicit real (a-h,l-z)

      integer neAlti,
     &        neMach,
     &        neFset
      real eAlti(neAlti),
     &     eMach(neMach), 
     &     eFset(neFset)

      include 'index.inc'
      include 'arrs.inc'
      include 'constants.inc'

      integer lu
      character*2 ctag

 1005 format(1x,a,a,a)
 1010 format(
     &/' =============================================================')
 1015 format(1x,'TASOPT v', f5.2)
 1020 format(
     &/' -----------------------------------------------------------')
 1040 format(
     &/' - - - - - - - - - - - - - - - - - - - - - - - - - - - - -')

      write(lu,1010)
      write(lu,1015) version
      write(lu,1020)
      write(lu,1005) 'Config:  ', configname
      write(lu,*)
      write(lu,1005) 'Case:  ', casename1
      write(lu,1005) '    :  ', casename2
      write(lu,1020)
      write(lu,1005) 'Engine off-design operation matrix'

c---- use the iptest operating point slot for all variables
      ip = iptest
      km = 1

c---- will use previously-set turbine cooling mass flow for all cases
      icool = 1

c========================================================
      do keAlti = 1, neAlti
        alt = eAlti(keAlti)

        write(lu,1010)
        write(lu,2010) 'altitude =', alt*ft_m, ' ft'
 2010   format(1x,a,f10.0, a)

        altkm = alt/1000.0
        call atmos(altkm, T0,p0,rho0,a0,mu0)

c--------------------------------------------------------
        do keMach = 1, neMach
          M0 = eMach(keMach)

          write(lu,1020)
          write(lu,2020) 'Mach =', M0
 2020     format(1x,a,f10.4)

c-------- initialize aero and engine state with chosen point ipinit
          ipinit = iptakeoff
          do ia = 1, iatotal
            para(ia,ip,km) = para(ia,ipinit,km)
          enddo
          do ie = 1, ietotal
            pare(ie,ip,km) = pare(ie,ipinit,km)
          enddo

c-------- set actual flight condition parameters
          para(iaalt ,ip,km) = alt
          para(iaMach,ip,km) = M0

          pare(iep0  ,ip,km) = p0
          pare(ieT0  ,ip,km) = T0
          pare(iea0  ,ip,km) = a0
          pare(ierho0,ip,km) = rho0
          pare(iemu0 ,ip,km) = mu0
          pare(ieM0  ,ip,km) = M0
          pare(ieu0  ,ip,km) = M0*a0

c-------- calculate thrust for max (takeoff) Tt4 at this altitude and Mach
          pare(ieTt4,ip,km) = pare(ieTt4,iptakeoff,km)
          icall = 1  ! specified Tt2
          inite = 1  ! will assume engine states are initialized
          call tfcalc(pari,parg,para(1,ip,km),pare(1,ip,km), ip,
     &             icall,icool,inite,
     &             ichoke5(ip,km),ichoke7(ip,km)) 

c-------- save max thrust value
          Fmax = pare(ieFe,ip,km)

c-------- initialize aero and engine state with roughly closest operating point
          if(eFset(1) .lt. 0.5) then
           ipinit = ipdescentn
          else
           ipinit = ipclimb1
          endif
          do ia = 1, iatotal
            para(ia,ip,km) = para(ia,ipinit,km)
          enddo
          do ie = 1, ietotal
            pare(ie,ip,km) = pare(ie,ipinit,km)
          enddo

c-------- set actual flight parameters again
          para(iaalt ,ip,km) = alt
          para(iaMach,ip,km) = M0

          pare(iep0  ,ip,km) = p0
          pare(ieT0  ,ip,km) = T0
          pare(iea0  ,ip,km) = a0
          pare(ierho0,ip,km) = rho0
          pare(iemu0 ,ip,km) = mu0
          pare(ieM0  ,ip,km) = M0
          pare(ieu0  ,ip,km) = M0*a0

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - 
          do keFset = 1, neFset
            frac = eFset(keFset)

            write(lu,1040)
            write(lu,2030) 'Fset =', frac*100.0
 2030       format(1x,a,f10.2, ' %')

            pare(ieFe,ip,km) = Fmax*frac
            icall = 2  ! specified Fe
            inite = 1  ! will assume engine states are initialized
            call tfcalc(pari,parg,para(1,ip,km),pare(1,ip,km), ip,
     &                  icall,icool,inite,
     &                  ichoke5(ip,km),ichoke7(ip,km))

            ione = mod( keFset , 10 )
            ctag = 'T' // char( ichar('0') + ione )
            call engwrt(lu,ctag,pare(1,ip,km))

          enddo ! next Fset
        enddo ! next Mach
      enddo ! next altitude

      return
      end ! eopwrt



      subroutine pltwrt(lu1,lu2,lu3,Lheader,
     &                  i,j,ni,nj,jdel,parsjspec)
      implicit real (a-h,l-z)

      include 'index.inc'
      include 'arrs.inc'
      include 'constants.inc'

      integer lu1,lu2,lu3
      logical Lheader
      integer i,j,ni,nj,jdel

      integer njd, ncol

      character*3 cmach, ci, cj
      character*128 fname

      integer iline, nline

      parameter(ixydim=15)
      integer nw,nh,nf,ne
      real xw(ixydim), yw(ixydim),
     &     xh(ixydim), yh(ixydim),
     &     xf(ixydim), yf(ixydim),
     &     xe(ixydim), ye(ixydim)

C---- create geometry-drawing arrays

      call airpic(pari, parg,
     &            nw, xw, yw,
     &            nh, xh, yh,
     &            nf, xf, yf,
     &            ne, xe, ye  )
      if(max(nw,nh,nf,ne).gt.ixydim) stop 'Array overflow on ixydim'


      km = 1
      ip = ipcruise1
      
      LoD = para(iaCL,ip,km)
     &    / para(iaCD,ip,km)
      cosL = cos(parg(igsweep) * pi/180.0)
      Mperp = para(iaMach,ip,km)*cosL
      CLperp = para(iaCL,ip,km)/cosL**2

 1000 format(a)

      if(Lheader) then
c----- write plot-label stuff only if requested
       nline = 19
       write(lu1,*) nline
       write(lu1,1000) pilabel
       write(lu1,1000) picolor

c----- x-axis labels for plots (j parameter)
       do iline = 1, nline
         write(lu1,1000) pjlabel
       enddo

c----- y-axis labels for plots (selected outputs)
       write(lu1,1000) '"PFEI    [KJ/kg-km]"'
       write(lu1,1000) '"2W_f     2W_{fmax}     W_0 [lb]"'
       write(lu1,1000) '"AR            L/D"'
       write(lu1,1000) '"CL           CLperp"'
       write(lu1,1000) '"t/c"'

       write(lu1,1000) '"alt.  [kft]"'
       write(lu1,1000) '"FPR"'
       write(lu1,1000) '"BPR"'
       write(lu1,1000) '"d_{fan}  [in]"'
       write(lu1,1000) '"W_{eng}  [lb]"'

       write(lu1,1000) '"F_{to}  [lb]"'
       write(lu1,1000) '"l_{to}        l_{bf}   [ft]"'
       write(lu1,1000) '"sin(gamma)  "'
       write(lu1,1000) '"V_{to}  [kt]"'
       write(lu1,1000) '"u_{fan}           u_{core}  [m/s]"'

       write(lu1,1000) '"dB_{FO}      dB_{CB}      dB_{SL}"'
       write(lu1,1000) '"T_{4 CR}         T_{4 TO}  [K]"'
       write(lu1,1000) '"BPR_{cool}"'
       write(lu1,1000) '"sweep  [deg]"'

c----- write output-file header only for first i,j point
       ncol = 27
       write(lu2,*) ni, nj, ncol

c----- write geometry-file header only for first i,j point
       njd = (nj-1)/jdel + 1
       write(lu3,*) ni, njd, nw, nh, nf, ne
      endif
C
c      write(*,*) parsj(j)

      km = 1
      ip = ipcruise1
      singBF = sin(parm(imgamVBF,km))
      write(lu2,2100) 
     &  parsjspec,
     &  parg(igPFEI),              ! 1
     &  2.0*parg(igWfuel)*lb_N,    ! 2
     &  2.0*parg(igWfmax)*lb_N,    ! 3
     &  parg(igWMTO)*lb_N,         ! 4
     &  parg(igAR),                ! 5
     &  LoD,                       ! 6
     &  parg(ighboxs),             ! 7
     &  para(iaCL,ip,km),          ! 8
     &  CLperp,                    ! 9
     &  para(iaalt,ip,km)*kft_m,   !10
     &  pare(iepif,ip,km),         !11
     &  pare(ieBPR,ip,km),         !12
     &  parg(igdfan)*in_m,         !13
     &  parg(igWeng)*lb_N,         !14
     &  parm(imFTO,km)*lb_N,       !15
     &  parm(imlTO,km)*ft_m,       !16
     &  parm(imlBF,km)*ft_m,       !17
     &  singBF,                    !18
     &  parm(imV2,km)*nmi_m/hr_s,  !19
     &  pare(ieu6,iptakeoff,km),   !20
     &  pare(ieu8,iptakeoff,km),   !21
     &  parm(imdBSL,km),           !22
     &  parm(imdBCB,km),           !23
     &  parm(imdBFO,km),           !24
     &  pare(iefc,iptakeoff,km),   !25
     &  pare(ieTt4,iptakeoff,km),  !26
     &  pare(ieTt4,ipcruise1,km),  !27
     &  parg(igsweep)              !28

 2100 format(1x,30g13.5)

      if(mod(j-1,jdel) .eq. 0) then
       do k=1, nw
         write(lu3,2100) xw(k)*ft_m, yw(k)*ft_m
       enddo
       do k=1, nh
         write(lu3,2100) xh(k)*ft_m, yh(k)*ft_m
       enddo
       do k=1, nf
         write(lu3,2100) xf(k)*ft_m, yf(k)*ft_m
       enddo

       xeng = parg(igxeng)
       yeng = parg(igyeng)

       xoff = xeng
       yoff = yeng
       do k=1, ne
         write(lu3,2100) (xoff+xe(k))*ft_m, (yoff+ye(k))*ft_m
       enddo
      endif

      return
      end ! pltwrt


      subroutine picwrt(lu,fname,pari,parg,xorg)
      implicit real (a-h,l-z)

      include 'index.inc'

      integer pari(iitotal)
      real parg(igtotal)

      integer lu
      character*(*) fname

      include 'constants.inc'

      parameter(ixydim=15)
      integer nw,nh,nf,ne
      real xw(ixydim), yw(ixydim),
     &     xh(ixydim), yh(ixydim),
     &     xf(ixydim), yf(ixydim),
     &     xe(ixydim), ye(ixydim)

      integer neng

C---- create geometry-drawing arrays

      call airpic(pari, parg,
     &            nw, xw, yw,
     &            nh, xh, yh,
     &            nf, xf, yf,
     &            ne, xe, ye  )
      if(max(nw,nh,nf,ne).gt.ixydim) stop 'Array overflow on ixydim'
c
      open(lu,file=fname,status='unknown')

 2100 format(1x,30g13.5)

      do k=1, nw
        write(lu,2100) yw(k)*ft_m, -(xw(k)-xorg)*ft_m
      enddo
      write(lu,*)
      do k=1, nw
        write(lu,2100) -yw(k)*ft_m, -(xw(k)-xorg)*ft_m
      enddo
      write(lu,*)

      do k=1, nh
        write(lu,2100) yh(k)*ft_m, -(xh(k)-xorg)*ft_m
      enddo
      write(lu,*)
      do k=1, nh
        write(lu,2100) -yh(k)*ft_m, -(xh(k)-xorg)*ft_m
      enddo
      write(lu,*)

      do k=1, nf
        write(lu,2100) yf(k)*ft_m, -(xf(k)-xorg)*ft_m
      enddo
      write(lu,*)
      do k=1, nf
        write(lu,2100) -yf(k)*ft_m, -(xf(k)-xorg)*ft_m
      enddo
      write(lu,*)

      xeng = parg(igxeng)
      yeng = parg(igyeng)
      neng = int(parg(igneng)+0.01)
      do ieng = 1, neng
        if(neng.eq.1) then
         frac = 0.0
        else
         frac = float(ieng-1)/float(neng-1)
        endif

        xoff = xeng
        yoffe = yeng*(1.0 - 2.0*frac)
        do k=1, ne
          write(lu,2100) (yoffe+ye(k))*ft_m, -(xoff+xe(k)-xorg)*ft_m
        enddo
        write(lu,*)
      enddo

      
      dy = 0.1*parg(igbo)*ft_m
      write(lu,2100)  2.0*dy, -(parg(igxNP)-xorg)*ft_m
      write(lu,2100) -2.0*dy, -(parg(igxNP)-xorg)*ft_m
      write(lu,*)
      write(lu,2100)  dy, -(parg(igxCGaft)-xorg)*ft_m
      write(lu,2100) -dy, -(parg(igxCGaft)-xorg)*ft_m
      write(lu,*)
      write(lu,2100)  dy, -(parg(igxCGfwd)-xorg)*ft_m
      write(lu,2100) -dy, -(parg(igxCGfwd)-xorg)*ft_m
      write(lu,*)
      write(lu,2100) 0.0, -(parg(igxCGfwd)-xorg)*ft_m
      write(lu,2100) 0.0, -(parg(igxCGaft)-xorg)*ft_m

      close(lu)

      return
      end ! picwrt


      subroutine picidr(lu,arg,i,j,ispars,jspars,pari,parg,xorg)
      implicit real (a-h,l-z)

      include 'index.inc'
      integer pari(iitotal)
      real parg(igtotal)

      integer lu
      character*(*) arg

      include 'constants.inc'

      parameter(ixydim=15)
      integer nw,nh,nf,ne
      real xw(ixydim), yw(ixydim),
     &     xh(ixydim), yh(ixydim),
     &     xf(ixydim), yf(ixydim),
     &     xe(ixydim), ye(ixydim)

      integer neng, narg, nfn


      character*3 ci, cj
      character*128 fname

      izero = ichar('0')

      idig1 = i / 10
      idig2 = i - idig1*10
      ci = 'i' // char(izero+idig1) // char(izero+idig2)

      idig1 = j / 10
      idig2 = j - idig1*10
      cj = 'j' // char(izero+idig1) // char(izero+idig2)

c---- construct output filename
      narg = index(arg,' ') - 1

      if(ispars .ne. 0 .and. jspars .ne. 0) then
       fname = arg(1:narg) // '_' // ci // '_' // cj // '.idraw'
      elseif(ispars .ne. 0) then
       fname = arg(1:narg) // '_' // ci // '.idraw'
      elseif(jspars .ne. 0) then
       fname = arg(1:narg) // '_' // cj // '.idraw'
      else
       fname = arg(1:narg) // '.idraw'
      endif
      open(lu,file=fname,status='unknown')
       nfn = index(fname,' ')
      write(*,*) 'Writing idraw  file:  ', fname(1:nfn)

c---- create geometry-drawing arrays
      call airpic(pari, parg,
     &            nw, xw, yw,
     &            nh, xh, yh,
     &            nf, xf, yf,
     &            ne, xe, ye  )
      if(max(nw,nh,nf,ne).gt.ixydim) stop 'Array overflow on ixydim'


      fac = ft_m * 1000.0

 2000 format(
     & /'Begin %I MLine'
     & /'%I b 65535'
     & /'0 0 0 [] 0 SetB'
     & /'%I cfg Black'
     & /'0 0 0 SetCFg'
     & /'%I cbg White'
     & /'1 1 1 SetCBg'
     & /'none SetP %I p n'
     & /'%I t'
     & /'[ 0.001 -0 -0 0.001 100 100 ] concat'
     & /'%I', i3)

 2100 format(1x,2i9)

 3000 format(
     &  i3,' MLine'
     & /'%I 1'
     & /'End' )

      write(lu,2000) nw
      do k=1, nw
        write(lu,2100) int(yw(k)*fac), int( -(xw(k)-xorg)*fac)
      enddo
      write(lu,3000) nw

      write(lu,2000) nw
      do k=1, nw
        write(lu,2100) int(-yw(k)*fac), int( -(xw(k)-xorg)*fac)
      enddo
      write(lu,3000) nw

      write(lu,2000) nh
      do k=1, nh
        write(lu,2100) int(yh(k)*fac), int( -(xh(k)-xorg)*fac)
      enddo
      write(lu,3000) nh

      write(lu,2000) nh
      do k=1, nh
        write(lu,2100) int(-yh(k)*fac), int( -(xh(k)-xorg)*fac)
      enddo
      write(lu,3000) nh

      write(lu,2000) nf
      do k=1, nf
        write(lu,2100) int(yf(k)*fac), int( -(xf(k)-xorg)*fac)
      enddo
      write(lu,3000) nf

      write(lu,2000) nf
      do k=1, nf
        write(lu,2100) int(-yf(k)*fac), int( -(xf(k)-xorg)*fac)
      enddo
      write(lu,3000) nf

      xeng = parg(igxeng)
      yeng = parg(igyeng)
      neng = int(parg(igneng)+0.01)
      do ieng = 1, neng
        if(neng.eq.1) then
         frac = 0.0
        else
         frac = float(ieng-1)/float(neng-1)
        endif

        xoff = xeng
        yoffe = yeng*(1.0 - 2.0*frac)

        write(lu,2000) ne
        do k=1, ne
          write(lu,2100) int((yoffe+ye(k))*fac), 
     &                   int(-(xoff+xe(k)-xorg)*fac)
        enddo
      write(lu,3000) ne

      enddo

      close(lu)

      return
      end ! picidr
