
      subroutine tfan(gee,M0,T0,p0,Mfan,Afan,     ! inputs
     &                BPR,pif,pic,pid,pib,        !
     &                Tt4,Ttf,ifuel,
     &                epolf,epolc,epolt,
     &                icool,
     &                Mtexit,Tmetal,dTstrk,Stc,
     &                M4a,ruc,
     &                BPRc,                       ! input or output
     &                TSFC,Fsp,hfuel,ff,mdot,     ! outputs
     &                Tt0,ht0,pt0,cpt0,Rt0,       !
     &                Tt2,ht2,pt2,cpt2,Rt2,
     &                Tt3,ht3,pt3,cpt3,Rt3,
     &                    ht4,pt4,cpt4,Rt4,
     &                Tt41,ht41,pt41,cpt41,Rt41,
     &                Tt5,ht5,pt5,cpt5,Rt5,
     &                Tt7,ht7,pt7,cpt7,Rt7,
     &                   u0,
     &                T6,u6,
     &                T8,u8,
     &                etaf,etac,etat )
c----------------------------------------------------------------
c     Turbofan performance routine.
c
c     Calculation procedure follows that of Kerrebrock,
c     but the usual gas property formulas are replaced
c     by subroutine calls, which can therefore implement
c     more general gas models.  
c     In addition, a turbine cooling model is added.
c
c     The gas routines reside in the following source files:
c      gascalc.f  Routines for various processes 
c                 (compressor, turbine, combustor, etc)
c      gasfun.f   Routines for computing cp(T), h(t), sigma(T), R,
c                 called by the routines in gascalc.f
c
c     Input
c     -----
c     gee    gravity acceleration
c     M0     freestream Mach
c     T0     freestream temperature  [K]
c     p0     freestream pressure  [Pa]
c     Mfan   fan-face Mach number, for computing engine mass flow
c     Afan   fan-face area [m^2] , for computing engine mass flow
c     BPR    bypass ratio  = mdot_fan/mdot_core
c     pif    fan      pressure ratio  ( = pt7/pt2)
c     pic    overall  pressure ratio  ( = pt3/pt2)
c     pid    diffuser pressure ratio  ( = pt2/pt0)
c     pib    burner   pressure ratio  ( = pt4/pt3)
c     Tt4    turbine-inlet total temperature [K]
c     Ttf    fuel temperature entering combustor
c     ifuel  fuel index, see subroutine gasfun (in gasfun.f)
c     epolf  fan        polytropic efficiency
c     epolc  compressor polytropic efficiency
c     epolt  turbine    polytropic efficiency
c
c     icool   turbine cooling flag
c              0 = no cooling, ignore all cooling parameters below
c              1 = usual cooling, using passed-in BPRc
c              2 = usual cooling, but set (and return) BPRc from Tmetal
c     Mtexit  turbine blade-row exit Mach, for setting temperature drops
c     Tmetal  specified metal temperature  [K], used only if icool=2
c     dTstrk  hot-streak temperature delta {K}, used only if icool=2
c     Stc     area-weighted Stanton number    , used only if icool=2
c     M4a     effective Mach at cooling-flow outlet (start of mixing)
c     ruc     cooling-flow outlet velocity ratio, u/ue
c     BPRc    cooling-flow bypass ratio, mdot_cool/mdot_core, input if icool=1
c
c     Output
c     ------
c     BPRc    cooling-flow bypass ratio, mdot_cool/mdot_core, output if icool=2
c     TSFC   thrust specific fuel consumption = mdot_fuel g / F   [1/s]
c     Fsp    specific thrust  = F / (mdot a) = F / ((1+BPR) mdot_core a)
c     hfuel  fuel heating value   [J / kg K]
c     ff     fuel mass flow fraction  =  mdot_fuel / mdot_core
c     mdot   core mass flow = mdot_core  [kg/s]
c     Tt?    total temperature
c     ht?    total complete enthalpy (includes heat of formation)
c     pt?    total pressure
c     cpt?   specific heat at stagnation temperature  (= dh/dT)
c     Rt?    gas constant  at stagnation conditions
c     T?     static temperature
c     u?     velocity
c     etaf   fan        overall efficiency
c     etac   compressor overall efficiency
c     etat   turbine    overall efficiency
c
c     The "?" symbol denotes the station index:
c       0  freestream
c       2  fan face
c       3  compressor exit
c       4  turbine inlet
c       5  turbine exit
c       6  turbine flow downstream
c       7  fan exit
c       8  fan flow downstream
c----------------------------------------------------------------
c
      implicit real (a-z)
      integer ifuel, icool

c---- stuff for calling mcool
      integer irow, nrow, nrowx
      parameter (nrowx=10)
      real Tgrow(nrowx), throw(nrowx), epsrow(nrowx)

c---- number of gas constituents
      integer n, nair
      parameter (n=6)
c
c---- mass fractions
      real alpha(n),   ! air
     &     beta(n),    ! fuel
     &     gamma(n),   ! combustion-caused change in air
     &     lambda(n),  ! combustion product gas
     &     lambdap(n)  ! combustion product gas with cooling flow added
c
c---- air fractions  N2      O2      CO2    H2O      Ar       fuel
      data alpha / 0.781 , 0.209 , 0.0004, 0.0   , 0.0096,  0.0 /
c
c---- fuel fractions
      data beta  / 0.0   , 0.0   , 0.0   , 0.0   , 0.0   ,  1.0 /
c
      data pi / 3.1415926535897932384626 /

c---- fan and turbine nozzle flow total-pressure ratios 
      pifnoz = 1.0
      pitnoz = 1.0

c---- number of air constitutents (all but fuel)
      nair = n-1

c===============================================================
c---- set combustion-change mass fractions gamma(i) for specified fuel
      call gasfuel(ifuel,gamma,n)
c
c===============================================================
c---- freestream static quantities
      call gassum(alpha,nair, T0, s0,dsdt,h0,dhdt,cp0,R0)
      gam0 = cp0/(cp0-R0)
      a0 = sqrt(gam0*R0*T0)
      u0 = M0*a0
c
c===============================================================
c---- freestream total quantities
      hspec = h0 + 0.5*u0**2
      Tguess = T0*(1.0 + 0.5*(gam0-1.0)*M0**2)
      call gas_tset(alpha,nair, hspec,Tguess, Tt0)
c 
      call gassum(alpha,nair, Tt0, st0,dsdt,ht0,dhdt,cpt0,Rt0)
      pt0 = p0 * exp((st0-s0)/Rt0)
c
c===============================================================
c---- diffuser flow 0..2
      Tt2 = Tt0
      st2 = st0
      ht2 = ht0
      cpt2 = cpt0
      Rt2 = Rt0
      pt2 = pt0 * pid
c
c===============================================================
c---- fan flow 2..7
      call gas_prat(alpha,nair, pt2,Tt2,ht2,st2,cpt2,Rt2, pif,epolf, 
     &                          pt7,Tt7,ht7,st7,cpt7,Rt7 )

c---- calculate fan efficiency (informative only -- not needed here)
      etaf = 0.
      call gas_prat(alpha,nair, pt2,Tt2,ht2,st2,cpt2,Rt2, pif,1.0, 
     &                          pt7i,Tt7i,ht7i,st7i,cpt7i,Rt7i )
      etaf = (ht7i-ht2)/(ht7-ht2)
c      write(*,*) 'eta_f', etaf, epolf
c
c===============================================================
c---- compressor flow 2..3
      call gas_prat(alpha,nair, pt2,Tt2,ht2,st2,cpt2,Rt2, pic,epolc,
     &                          pt3,Tt3,ht3,st3,cpt3,Rt3)
cc    write(*,*) 'epolc Tt3', epolc, Tt3

c---- calculate compressor efficiency (informative only -- not needed here)
      etac = 0.
      call gas_prat(alpha,nair, pt2,Tt2,ht2,st2,cpt2,Rt2, pic,1.0,
     &                          pt3i,Tt3i,ht3i,st3i,cpt3i,Rt3i)
      etac = (ht3i-ht2)/(ht3-ht2)
cc      write(*,*) 'eta_c', etac, epolc

c===============================================================
c---- combustor flow 3..4   (ffb = mdot_fuel/mdot_burner)
      call gas_burn(alpha,beta,gamma,n,ifuel, Tt3,Ttf,Tt4, ffb,lambda)
      call gassum(lambda,nair, Tt4, st4,dsdt,ht4,dhdt,cpt4,Rt4)
      pt4 = pt3*pib
      gam4 = cpt4/(cpt4-Rt4)

c---- set ff = mdot_fuel/mdot_core = ffb * mdot_burner/mdot_core
      ff = ffb*(1.0-BPRc)

c      write(*,*) 'ff    Tt4', ff, Tt4
c      write(*,12) 'pt0 pt2 pt3 pt4', p0, pt0, pt2, pt3, pt4


c     write(*,'(1x,8f9.5)') alpha,
c    & alpha(1)+alpha(2)+alpha(3)+alpha(4)+alpha(5)+alpha(6)
c     write(*,'(1x,8f9.5)') gamma,
c    & gamma(1)+gamma(2)+gamma(3)+gamma(4)+gamma(5)+gamma(6)
c     write(*,'(1x,8f9.5)') lambda,
c    & lambda(1)+lambda(2)+lambda(3)+lambda(4)+lambda(5)+lambda(6)

c===============================================================
      if(icool.eq.2) then
c----- set cooling mass flow
       efilm = 0.7
       tfilm = 0.4
c
       gmi4 = Rt4/(cpt4-Rt4)
       Trrat = 1.0 / (1.0 + 0.5*gmi4*Mtexit**2)

c----- calculate cooling mass flow ratio BPRc
       call mcool(Tmetal,Tt3,Tt4,dTstrk, Trrat,
     &            efilm,tfilm,Stc,
     &            throw, epsrow, BPRc, nrow, nrowx)

      else
c----- BPRc is assumed to be passed in
      endif

c----------------------------------------------------------------
      if(icool.eq.0) then
c----- no cooling air present
       pt41 = pt4
       Tt41 = Tt4
       ht41 = ht4
       st41 = st4
       cpt41 = cpt4
       Rt41 = Rt4
       do i = 1, nair
         lambdap(i) = lambda(i)
       enddo
c
c----------------------------------------------------------------
      else
       pt4a = pt4  
       Tt4a = Tt4  
       ht4a = ht4  
       st4a = st4  
       cpt4a= cpt4 
       Rt4a = Rt4  

c----- speed at start-of-mixing station 4a
       call gas_mach(lambda,nair,
     &               pt4a,Tt4a,ht4a,st4a,cpt4a,rt4a, 0.0,M4a, 1.0,
     &               p4a ,T4a ,h4a ,s4a ,cp4a ,r4a)
       u4sq = max( 2.0*(ht4a-h4a) , 0.0 )
       u4a = sqrt(u4sq)

       uc = ruc*u4a
c---------------------------------------------------------
c----- IGV exit mixing
       frac4 = (1.0-BPRc+ff)/(1.0+ff)
       fracm =      BPRc    /(1.0+ff)

c----- mixed constituent fraction vector from mass equation
       do i = 1, nair
         lambdap(i) = frac4*lambda(i) + fracm*alpha(i)
       enddo

c----- mixed total enthalpy from enthalpy equation
       ht41 = frac4*ht4 + fracm*ht3

c----- total temperature from total enthalpy
       Tguess = frac4*Tt4 + fracm*Tt3
       call gas_tset(lambdap,nair, ht41,Tguess, Tt41)

c----- all total quantities, from total temperature
       call gassum(lambdap,nair, Tt41, st41,dsdt,ht41,dhdt,cpt41,Rt41)
       pt41 = pt4

c----- mixed velocity from momentum equation
       p41 = p4a
       u41 = frac4*u4a + fracm*uc
c
c----- static temperature from static enthalpy
       h41 = ht41 - 0.5*u41**2
       Tguess = t4a + (h41-h4a)/cp4a
       call gas_tset(lambdap,nair, h41,Tguess, T41)
c
c----- all static quantities, from static temperature
       call gassum(lambdap,nair, T41, s41,dsdt,h41,dhdt,cp41,R41)

c----- all stagnation quantities, from total-static enthalpy difference
       delh = ht41 - h41
       epi = 1.0
       call gas_delh(lambdap,nair,
     &                p41 ,T41 ,h41 ,s41 ,cp41 ,R41 ,delh,epi,
     &                pt41,Tt41,ht41,st41,cpt41,Rt41)

c       write(*,*)
c       write(*,12) 'u4a u41 ', u4a, u41
c       write(*,12) 'uc      ', uc
c       write(*,*)
ccc    write(*,12) 'ht3 ht4 ht4a ht41', ht3, ht4, ht4a, ht41
c       write(*,12) 'Tt3 Tt4 Tt4a Tt41', Tt3, Tt4, Tt4a, Tt41
c       write(*,12) 'pt3 pt4 pt4a pt41', pt3, pt4, pt4a, pt41
c 12    format(1x,a,8g12.4)
      endif
c----------------------------------------------------------------

c===============================================================
c---- turbine flow 4..5
      delh = -(ht3-ht2 + BPR*(ht7-ht2)) / (1.0 + ff)
      epi = 1.0/epolt
      call gas_delh(lambdap,nair,
     &                pt41,Tt41,ht41,st41,cpt41,Rt41, delh,epi,
     &                pt5 ,Tt5 ,ht5 ,st5 ,cpt5 ,Rt5  )

cc    write(*,*) 'delh     ', delh
cc    write(*,*) 'epolt Tt5', epolt, Tt5

c---- calculate turbine efficiency (informative only -- not needed here)
      etat = 0.
      pit = pt5/pt41
      call gas_prat(lambdap,nair,pt41,Tt41,ht41,st41,cpt41,Rt41,pit,1.0,
     &                           pt5i,Tt5i,ht5i,st5i,cpt5i,Rt5i)
      etat = (ht5-ht41)/(ht5i-ht41)
cc      write(*,*) 'eta_t', etat, epolt

c
c===============================================================
c---- fan nozzle flow 7..8, use alpha mass fraction (air)
      pt8 = pt7*pifnoz
      ht8 = ht7
      Tt8 = Tt7
      st8 = st7
      cpt8 = cpt7
      Rt8 = Rt7
      pratfn = p0/pt8
      call gas_prat(alpha,nair, pt8,Tt8,ht8,st8,cpt8,Rt8, pratfn,1.0,
     &                          p8 ,T8 ,h8, s8, cp8 ,R8 )
      u8 = sqrt(2.0*(ht8-h8))

cc    write(*,*) 'Pt8   u8 ', pt8, u8

c
c---- fan specific thrust = F_fan / mdot_core a
      F8sp = BPR*(u8 - u0)/a0
c
c===============================================================
c---- turbine nozzle flow 5..6, use lambda mass fraction (combustion products)
      pt6 = pt5*pitnoz
      ht6 = ht5
      Tt6 = Tt5
      st6 = st5
      cpt6 = cpt5
      Rt6 = Rt5
      prattn = p0/pt6
      call gas_prat(lambdap,nair, pt6,Tt6,ht6,st6,cpt6,Rt6, prattn,1.0,
     &                            p6 ,T6 ,h6, s6 ,cp6 ,R6 )
      h6 = min( h6 , ht6 )
      u6 = sqrt(2.0*(ht6-h6))

cc    write(*,*) 'Pt6   u6 ', pt6, u6
c
c---- core specific thrust = F_core / mdot_core a
      F6sp = ((1.0+ff)*u6 - u0)/a0
c
c===============================================================
c---- overall specific thrust  Fsp  =  F / (mdot_fan + mdot_core) a
      Fsp = (F6sp + F8sp)/(1.0+BPR)

c---- overall specific impulse and fuel consumption
      Isp = Fsp*a0*(1.0+BPR)/(gee*ff)
      TSFC = 1.0/Isp
c
      cpa = 0.5*(cpt3+cpt41)
      hfuel = ((Tt41-Tt3) + ff*(Tt41-Ttf))*cpa / ff
c
c===============================================================
c---- actual core mass flow for specified Afan = A2
      if(Afan .gt. 0.0) then
       M2 = Mfan
       call gas_mach(alpha,nair, pt2,tt2,ht2,st2,cpt2,rt2, 0.0, M2, 1.0,
     &                           p2 ,t2 ,h2 ,s2 ,cp2 ,r2 )
       u2 = sqrt(2.0*(ht2-h2))
       rho2 = p2/(r2*t2)
       mdot = rho2*u2*Afan / (1.0+BPR)
      endif

      return
      end ! tfan



      subroutine mcool(Tmetal,Tt3,Tt4,dTstreak, Trrat,
     &                 efilm,tfilm,Stc,
     &                 throw, epsrow, BPRc, nrow, nrowx)
c==============================================================
c     Calculates cooling mass flow requirement.
c     All temperatures are in Kelvin.
c
c     Input
c     -----
c     Tmetal    design metal temperature
c     Tt3       cooling flow temperature
c     Tt4       hot gas temperature from burner
c     dTstreak  added temperature seen by first IGV
c     Trrat     static temperature ratio across each blade row, T4.1 / T4
c     
c     Output
c     ------
c     throw(.)  cooling effectiveness for each blade row 1,2,3...nrow
c     epsrow(.) cooling mass flow ratio for each blade row, m_c_row/m_core
c     BPRc      total cooling mass flow for all rows with cooling
c     nrow      number of blade rows which need cooling
c     nrowx     dimension of arrays
c
c==============================================================
      real throw(*), epsrow(*)
c
      do irow = 1, nrowx
        if(irow.eq.1) then
         Tg = Tt4 + dTstreak
        else
         Tg = Tt4 * Trrat**(irow-1)
        endif
c
        throw(irow) = (Tg-Tmetal)/(Tg-Tt3)
        eps0 = Stc*(throw(irow)*(1.0-efilm*tfilm) - tfilm*(1.0-efilm))
     &             /(efilm*(1.0-tfilm))
        if(eps0 .lt. 0.0) go to 5
        nrow = irow
        epsrow(irow) = eps0/(1.0 + eps0)
      enddo
 5    continue
     
      BPRc = 0.
      do irow = 1, nrow
        BPRc = BPRc + epsrow(irow)
      enddo

      return
      end ! mcool
