
      subroutine mcool(ncrowx,ncrow,
     &                 Tmrow,Tt3,Tt4,dTstreak, Trrat,
     &                 efilm,tfilm,StA, 
     &                 epsrow, epsrow_Tt3, epsrow_Tt4, epsrow_Trr)
    
c==============================================================
c     Calculates cooling mass flow requirement.
c
c  Input
c  -----
c  ncrowx   dimension of Tmrow(.),epsrow(.) arrays (max number of blade rows)
c  Tmrow(.) design metal temperature for each blade row
c  Tt3      cooling flow temperature
c  Tt4      hot gas temperature from burner
c  dTstreak hot-streak temperature increase over Tt4, for first blade row 
c  Trrat    static temperature ratio across each blade row, T4.1 / T4
c  efilm    cooling efficiency = (Tco-Tci)/(Tmetal-Tci)
c  tfilm    film effectiveness = (Tgas-Tfaw)/(Tgas-Tco)
c            Tco = temperature of cooling air exiting  blade
c            Tci = temperature of cooling air entering blade
c            Tfaw = film adiabatic wall temperature (for insulated-wall case)
c  StA      area-weighted external Stanton number = St (Asurf/Aflow) cpgas/cpcool
c
c  Output
c  ------
c  ncrow     number of blade rows which need cooling
c  epsrow(.) cooling mass flow ratio for each blade row, m_c_row/m_air
c
c==============================================================
      real Tmrow(ncrowx),
     &     epsrow(ncrowx), 
     &     epsrow_Tt3(ncrowx),
     &     epsrow_Tt4(ncrowx),
     &     epsrow_Trr(ncrowx)

c---- assume zero cooling mass flow for all blade rows
      do icrow = 1, ncrowx
        epsrow(icrow)     = 0.
        epsrow_Tt3(icrow) = 0.
        epsrow_Tt4(icrow) = 0.
        epsrow_Trr(icrow) = 0.
      enddo

c---- step downstream, until cooling is no longer needed
      ncrow = 0
      do icrow = 1, ncrowx
        if(icrow.eq.1) then
         Tg     = Tt4 + dTstreak
         Tg_Tt4 = 1.0
         Tg_Trr = 0.
        else
         Tg     = Tt4*Trrat**(icrow-1)
         Tg_Tt4 =     Trrat**(icrow-1)
         Tg_Trr = float(icrow-1)*Tg/Trrat
        endif
c
c------ cooling effectiveness for this blade row
        theta = (Tg-Tmrow(icrow))/(Tg-Tt3)
        theta_Tt3 =      theta /(Tg-Tt3)
        theta_Tt4 = (1.0-theta)/(Tg-Tt3) * Tg_Tt4
        theta_Trr = (1.0-theta)/(Tg-Tt3) * Tg_Trr

        eps0       = StA*(theta*(1.0-efilm*tfilm) - tfilm*(1.0-efilm))
     &                 / (efilm*(1.0-theta))
        eps0_theta = StA       *(1.0-efilm*tfilm)
     &                 / (efilm*(1.0-theta))
     &             +  eps0/(1.0-theta)
        eps0_Tt3 = eps0_theta*theta_Tt3
        eps0_Tt4 = eps0_theta*theta_Tt4
        eps0_Trr = eps0_theta*theta_Trr

        if(eps0 .lt. 0.0) go to 5
        ncrow = icrow
        epsrow(icrow) = eps0/(1.0 + eps0)
        epsrow_eps0 = (1.0 - epsrow(icrow))/(1.0 + eps0)

        epsrow_Tt3(icrow) = epsrow_eps0*eps0_Tt3
        epsrow_Tt4(icrow) = epsrow_eps0*eps0_Tt4
        epsrow_Trr(icrow) = epsrow_eps0*eps0_Trr

c        write(*,'(1x,a,i3,8g13.5)') 'irow Tg Tm Tt3 theta eps',
c     &      icrow, Tg, Tmrow(icrow), Tt3, theta, epsrow(icrow)

      enddo
 5    continue
     
      return
      end ! mcool



      subroutine Tmcalc(ncrowx,ncrow,
     &                  Tmrow,Tt3,Tt4,dTstreak, Trrat,
     &                  efilm,tfilm,StA, epsrow)
c==============================================================
c     Calculates metal temperature for blade row
c
c  Input
c  -----
c  ncrowx   dimension of Tmrow(.),epsrow(.) arrays (max number of blade rows)
c  ncrow    number of blade rows which are cooled
c  epsrow(.) cooling mass flow ratio for each blade row, m_c_row/m_air
c  Tt3      cooling flow temperature
c  Tt4      hot gas temperature from burner
c  dTstreak hot-streak temperature increase over Tt4, for first blade row 
c  Trrat    static temperature ratio across each blade row, T4.1 / T4
c  efilm    cooling efficiency = (Tco-Tci)/(Tmetal-Tci)
c  tfilm    film effectiveness = (Tgas-Tfaw)/(Tgas-Tco)
c            Tco = temperature of cooling air exiting  blade
c            Tci = temperature of cooling air entering blade
c            Tfaw = film adiabatic wall temperature (for insulated-wall case)
c  StA      area-weighted external Stanton number = St (Asurf/Aflow) cpgas/cpcool
c
c  Output
c  ------
c  Tmrow(.) design metal temperature for each blade row
c
c==============================================================
      real Tmrow(ncrowx), epsrow(ncrowx)
c
      do icrow = 1, ncrow
        if(icrow.eq.1) then
         Tg = Tt4 + dTstreak
        else
         Tg = Tt4 * Trrat**(icrow-1)
        endif

        eps = epsrow(icrow)
        theta = (eps*efilm + StA*tfilm*(1.0-efilm)*(1.0-eps))
     &        / (eps*efilm + StA*(1.0-efilm*tfilm)*(1.0-eps))

        Tmrow(icrow) = Tg - (Tg-Tt3)*theta

        theta = (Tg-Tmrow(icrow))/(Tg-Tt3)

c        write(*,'(1x,i3,8g13.5)') 
c     &      icrow, Tg, Tmrow(icrow), Tt3, theta, epsrow(icrow)

      enddo

      return
      end ! Tmcalc
