      subroutine tfnoise(
     &       x,y,z,
     &       climb,alpha,vector, 
     &       rho0, p0, T0, mu0, c0,
     &       A6,A8,u6,u8,T6,T8,M0,
     &       etaf,FPR,mdot,
     &       BPR,Mtrd,Mtr,Mt,RPM,
     &       rss,B,V,htr,
     &       neng,type,method,
     &       spl_jet_dba,
     &       spl_f_t_dba,
     &       spl_f_b_dba,
     &       spl_f_z_dba,
     &       spl_r_t_dba,
     &       spl_r_b_dba,
     &       spl_tot_dba )
c---------------------------------------------------------------------------------
c This function predicts the maximum noise from jet noise+fan noise 
c in dBA at sideline position
c
c   x,y,z axes use standard S&C sign conventions:
c    + x  forward
c    + y  right
c    + z  down
c
c INPUTS
c   x     observer axial position     [m]   (+ if ahead of airplane)
c   y     observer sideline position  [m]   (+ if to right of airplane)
c   z     observer vertical position  [m]   (+ if below airplane)
c   climb  climb angle          [rad]
c   alpha  angle of attack      [rad]
c   vector thrust-vector angle  [rad]
c   rho0  air density
c   p0    air pressure
c   T0    air temperature
c   mu0   air viscosity
c   c0    air speed of sound
c   A6    core nozzle area      [m^2]
c   A8    fan  nozzle area      [m^2]
c   u6    core nozzle velocity  [m/s]
c   u8    fan  nozzle velocity  [m/s]
c   T6    core jet static temperature  [K]
c   T8    fan  jet static temperature  [K]
c   M0    flight Mach number
c   etaf  fan efficiency
c   FPR   fan pressure ratio
c   mdot  core mass flow         [kg/s]
c   BPR   bypass ratio
c   Mt    fan tip Mach number
c   Mrt   relative flow Mach number at the tip
c   Mrtd  relative flow Mach number at the tip at design conditions
c   RPM   fan rotational speed   [revs/min]
c   rss   Rotor Stator Spacing  [mm]
c   B     Number rotor blades
c   V     Number stator blades
c   htr   Hub tip ratio
c   neng  number of engines
c   type  = 0  unmixed exhaust
c         = 1  mixed   exhaust
c         If mixed flow, the quantities with subscript "6" correspond 
c         to the jet conditions and "8" would be zero
c   method = 1 ! ESDU method
c   method = 2 ! Heidmann's method (part of ANOPP)
c   method = 3 ! Heidmann's method with Allied Signal modifications 
c                ! (also known as small engine method and part of ANOPP)
c
c OUTPUTS 
c    spl_jet_dba
c    spl_f_t_dba
c    spl_f_b_dba
c    spl_f_z_dba
c    spl_r_t_dba
c    spl_r_b_dba
c    spl_tot_dba
c- - - - - - - - - - - - - - - - - - - - - - - - - - - -
c   NO SIGNIFICANT DIFFERENCES BETWEEN THE METHODS
c   AROUND 1DBA WHEN CALCULATED NOISE WITH B737-300/CFM56, B777-200/GE90.
c    Chose method 2 as is part of ANOPP 
c- - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
c OUTPUTS
c
c   NOISE_MAX(1)  Jet                   
c   NOISE_MAX(2)  Forward Fan Tonal     
c   NOISE_MAX(3)  Forward Fan Broadband 
c   NOISE_MAX(4)  Forward Fan Buzzsaw   
c   NOISE_MAX(5)  Rearward Fan Tonal    
c   NOISE_MAX(6)  Rearward Fan Broadband
c   NOISE_MAX(8)  Total
c
c The results considers atmospheric attenuation and ground reflection. 
c Atmospheric attenuation is a fixed value frequency dependent
c Ground reflection is added as a +3dB
c It does not consider lateral attenuation or Doppler corrections. 
c The shielding is NOT added at the moment
c---------------------------------------------------------------------------------
      implicit real(a-z)
      integer type, method

      data pi / 3.1415926535897932384626 /

c---- distance to observer
      Dist = sqrt(x**2 + y**2 + z**2)

c---- angle of engine axis from horizontal
      te = climb + alpha + vector

c---- observer position in engine axes
      xe = x*cos(te) - z*sin(te)
      ye = y
      ze = z*cos(te) + x*sin(te)

c---- directivity angle
      theta = atan2( sqrt(ye**2 + ze**2) , xe )

c---- angle of flight direction from horizontal
      tf = climb

c---- observer position in flight-path axes
      xf = x*cos(tf) - z*sin(tf)
      yf = y
      zf = z*cos(tf) + x*sin(tf)

c---- modified directivity angle
      psi = atan2( sqrt(yf**2 + zf**2) , xf )

c      write(*,*)
c      write(*,*) 'theta ', theta * 180.0/pi
c      write(*,*) 'alpha ', alpha * 180.0/pi
c      write(*,*) 'psi   ', psi   * 180.0/pi
c      write(*,*) 'x', x
c      write(*,*) 'y', y
c      write(*,*) 'z', z

c-----------------------------------------------------------------
c---- jet noise
      call jet_noise(Dist,theta,alpha,psi,
     &               rho0,p0,T0,mu0,c0,
     &               A6,A8,u6,u8,T6,T8,M0,type,
     &               spl_jet_dba)

      spl_jet_dba = spl_jet_dba + 10.0*log10(neng)

c-----------------------------------------------------------------
c---- fan noise
c     The data for the fan geometry comes from the SAX-40 fan design by Dan Chriton
c      rss = 302.0  ! Rotor Stator Spacing
c      B   = 20.0   ! Number rotor blades
c      V   = 44.0   ! Number stator blades
c      htr = 0.29   ! Hub tip ratio

c   If distributed propulsions, assumed that all fans are operating at same 
c   point on the characteristics so all produce the same noise
      call esdu_fan_noise_total(
     &            method,Dist,theta,rho0,p0,T0,mu0,c0,
     &            M0,etaf,FPR,
     &            rss,B,V,mdot,BPR,Mtrd,Mtr,Mt,RPM,
     &            spl_f_t_dba,
     &            spl_f_b_dba,
     &            spl_f_z_dba,
     &            spl_r_t_dba,
     &            spl_r_b_dba )

      spl_f_t_dba = spl_f_t_dba + 10.0*log10(neng)
      spl_f_b_dba = spl_f_b_dba + 10.0*log10(neng)
      spl_f_z_dba = spl_f_z_dba + 10.0*log10(neng)
      spl_r_t_dba = spl_r_t_dba + 10.0*log10(neng)
      spl_r_b_dba = spl_r_b_dba + 10.0*log10(neng)

c-------------------------------------------------------------------
c---- total noise
      nsum = 10.0**(spl_jet_dba/10.0)
     &     + 10.0**(spl_f_t_dba/10.0)
     &     + 10.0**(spl_f_b_dba/10.0)
     &     + 10.0**(spl_f_z_dba/10.0)
     &     + 10.0**(spl_r_t_dba/10.0)
     &     + 10.0**(spl_r_b_dba/10.0)
      spl_tot_dba = 10.0 * log10(nsum)


      return
      end ! tfnoise



      subroutine jet_noise(Dist,theta,alpha,psi,
     &                    rho0,p0,T0,mu0,c0,
     &                    A6,A8,u6,u8,T6,T8, M0,type,
     &                    spl_jet)
c--------------------------------------------------------------------------
c This function estimates jet noise. Same model that was used by Chriton.
c Modifications to include two type of engines: mixed and unmixed exhaust
c
c The correlation to calculate the noise at a observer located at a known
c position from the source is coming from Stone et al. (AIAA journal Vol.21,
c No. 3). The correlation gives the results propagated to the ground and 
c as a function of the polar angle with flight corrections from Low 1994.
c
c The spectra is added based on values from Table 1 in the paper
c
c Lateral attenuation has not been included
c Atmospheric attenuation is included as a fixed correction
c Ground reflection is included adding +3dB to the noise values
c
c This gives values of SPL (dB) at different frequencies and at different
c polar angles. The SPL(dB) is converted into dBA by applying a correction.
c
c Then, noise is integrated in all the frequency spectra 
c 
c INPUTS
c   Dist   distance to observer
c   theta  angle from jet axis
c   alpha  jet angle of attack (for effective turned exhaust calculation)
c   psi    angle from airplane-path axis
c   rho0   atm density
c   p0     atm pressure
c   T0     atm temperature
c   mu0    atm viscosity
c   c0     atm speed of sound
c   A6     core nozzle area
c   A8     fan  nozzle area
c   u6     core nozzle velocity
c   u8     fan  nozzle velocity
c   T6     core jet static temperature
c   T8     fan  jet static temperature
c   M0     flight Mach number
c   type    = 0  unmixed
c           = 1  mixed
c
c OUTPUTS
c   jet_noise   jet noise in dBA
c
c-----------------------------------------------------------------------------
      implicit real(a-z)
      integer type
      integer i

      include 'freq.inc'

      data pi / 3.1415926535897932384626 /

c---- Geometry of the engine
c-    Hydraulic diameter. Considering round nozzle so
c-    hydraulic diameter is the same as the nozzle diameter
      Dh = sqrt(4.0*A6/pi)

c---- ISA conditions
      R = 287.0
      gamma = 1.4
      Tisa  = 288.0
      Pisa  = 101325.0
      rhoisa = Pisa/(R*Tisa)
      cisa  = sqrt(gamma*R*Tisa)

c---- Flight speed
      u0 = M0*sqrt(gamma*R*T0)

c---- Effective velocity
      udnorm = max( 1.0 - (u0/u6)*cos(alpha) , 0.0001 )
      ue = u6*udnorm**(2.0/3.0)

c---- Inner Jet Mach number, total temperature, density
      M6 = u6/sqrt(gamma*R*T6)
      Tt6 = T6*(1.0 + 0.5*(gamma-1.0)*M6**2)
      rho6 = p0/(R*T6)

c---- Outer Jet Mach number, total temperature and density
      if(type .eq. 0) then
c----- unmixed
       M8 = u8/sqrt(gamma*R*T8)
       Tt8 = T8*(1.0 + 0.5*(gamma-1.0)*M8**2)
       rho8 = p0/(R*T8)
      endif

c---- Density exponent with flight corrections
      ggmi = gamma / (gamma-1.0)
      w = (3.0*(ue/c0)**ggmi) / (0.6 + (ue/c0)**ggmi) - 1.0

c---- Convective mach number with flight corrections
cc    Mc = 0.62*(u8-u0*cos(alpha))/c0   !%% BUG???
      Mc = 0.62*(u6-u0*cos(alpha))/c0

c---- Corrected directivity angle for spectra calculations
      thetap = theta*(u6/c0)**0.1

      cost  = cos(theta )
      costp = cos(thetap)
      cosp  = cos(psi   )

      Mcosp = min( M0*cosp , 0.99 )

c---- CALCULATION OF NOISE. 
c      This is OSPL. 
c      This give the peak value no spectra and as a function 
c      of the polar angle. For sideline in this calculation, 
c      there is just one value of the polar angle
      UOL1 = 141.0
     &     + 10.0*log10((rho0/rhoisa)**2*(c0/cisa)**4)
     &     + 10.0*log10(A6/Dist**2)
     &     + 10.0*log10((rho6/rho0)**w)
     &     + 10.0*log10((ue/c0)**7.5)
     &     - 15.0*log10((1.0+Mc*cost)**2 + 0.04*Mc**2)
     &     - 10.0*log10(1.0 - Mcosp)
     &     +  3.0*log10(2.0*A6/(pi*Dh**2) + 0.5)

c     write(*,*) (rho0/rhoisa)**2*(c0/cisa)**4
c     write(*,*) A6/Dist**2
c     write(*,*) (rho6/rho0)**w
c     write(*,*) (ue/c0)**7.5
c     write(*,*) (1.0+Mc*cost)**2 + 0.04*Mc**2
c     write(*,*) 1.0 - M0*cosp
c     write(*,*) 2.0*A6/(pi*Dh**2) + 0.5
c     write(*,*)

c---- unmodified time scale for Strouhal number calculation
      tS1 = sqrt(4.0*A6/pi)/ue
     &   * (Dh/(sqrt(4*A6/pi)))**0.4
     &   * (Tt6/T0)**(0.4*(1.0 + costp))
     &   * (1.0 - M0*cosp)
     &   * ( (1.0 + 0.62*((u6-u0)/c0)*cost)**2
     &         + 0.01538*((u6-u0)/c0)**2 )**0.5
     &   / ( (1.0 + 0.62*( u6    /c0)*cost)**2
     &         + 0.01538*( u6    /c0)**2 )**0.5

c     write(*,*) sqrt(4.0*A6/pi)/ue
c     write(*,*) (Dh/(sqrt(4*A6/pi)))**0.4
c     write(*,*)  (Tt6/T0)**(0.4*(1.0 + costp))
c     write(*,*)  (1.0 - M0*cosp)
c     write(*,*)  ( (1.0 + 0.62*((u6-u0)/c0)*cost)**2
c    &                + 0.01538*((u6-u0)/c0)**2 )**0.5
c     write(*,*)  ( (1.0 + 0.62*( u6    /c0)*cost)**2
c    &                + 0.01538*( u6    /c0)**2 )**0.5
c     write(*,*)

      Arat = A8/A6
      urat = u8/u6
      Trat = Tt8/Tt6

      if(type .eq. 0) then
c----- unmixed... modify noise and time scale for coax jet configuration
       if(Arat .lt. 29.7) then
        m = 1.1*sqrt(Arat)
       else
        m = 6.0
       endif
       arg = max( 1.0-urat , 0.001 ) ** m
     &     + 1.2*( (1.0 + Arat*urat**2)**4
     &            /(1.0 + Arat)**3 )
       UOL = UOL1
     &     -  5.0*log10(Trat)
     &     + 10.0*log10(arg)
       Fs = fsfun(Arat,urat)

       TuArat = Trat*urat*Arat
       tmp = 0.5 + TuArat / (1.0 + TuArat)
       tS = tS1 / (1.0 - Fs*tmp**2)
      else
c----- mixed... use noise and time scale as is
       UOL = UOL1
       tS = tS1
      endif

      tS = max( tS , 1.0e-6 )

c---- calculate dBA for range of frequencies, and integrate over the frequencies
      SPLsum = 0.
      do i = 1, nfreq
c------ Strouhal number for this frequency
        S = freq(i)*tS

c------ correction for frequency
        dBfreqcorrection = fpfun(S,thetap)

c------ correction for A-weighting, atmospheric attenuation
c        ( dBatten is per 100m )
        SPLdBAcorrected = UOL + 3.0
     &                  + dBfreqcorrection
     &                  + dBAcorr(i)
     &                  + dBatten(i) * Dist/100.0

c------ accumulate integration over the frequency range
        SPLsum = SPLsum + 10.0**(SPLdBAcorrected/10.0)
      enddo

      spl_jet = 10.0*log10(SPLsum)

      return
      end ! jet_noise


      subroutine esdu_fan_noise_total(
     &       method,Dist,theta,
     &       rho0,p0,T0,mu0,c0,
     &       M0,etaf,FPR,rss,B,V,mdot,BPR,
     &       Mtrd,Mtr,Mt,RPM,
     &       spl_f_t_dba, 
     &       spl_f_b_dba, 
     &       spl_f_z_dba, 
     &       spl_r_t_dba, 
     &       spl_r_b_dba  )
c----------------------------------------------------------------------------
c This function estimates the fan noise, both, rearward and forward. 
c In both cases there is a broadband noise component and a discrete tone
c component. There is also an addtional noise component that is the
c combination tone or "buzz-saw" component when the relative tip speed 
c is supersonic.
c
c Description of the input parameters
c
c   Dist   distance between the aircraft and observer
c   theta  angle(s) from engine axis
c   rho0   atm density
c   p0     atm pressure
c   T0     atm temperature
c   mu0    atm viscosity
c   c0     atm speed of sound
c   M0     flight mach number
c   etaf   fan efficiency
c   FPR    fan pressure ratio
c   rss    rotor stator spacing
c   B      number of rotor blades
c   V      number of stator blades
c   mdot   core mass flow
c   BPR    bypass ratio
c   Mt     tip mach number
c   Mrt    relative flow mach number at the tip
c   Mrtd   relative flow mach number at the tip at design
c   RPM    fan rotational speed
c
c----------------------------------------------------------------------------
      implicit real (a-z)
      integer method

      integer nfmax, i, shocklocation

      integer nf_bb_fore,
     &        nf_bb_rear,
     &        nf_t_fore,
     &        nf_t_rear,
     &        nf_buzz

c---- nfmax must be at least as large as nfreq defined in freq.inc
      parameter (nfmax=24)

      real spl_bb_fore(nfmax), freq_bb_fore(nfmax),
     &     spl_bb_rear(nfmax), freq_bb_rear(nfmax),
     &     spl_t_fore(nfmax), freq_t_fore(nfmax),
     &     spl_t_rear(nfmax), freq_t_rear(nfmax),
     &     spl_buzz(nfmax), freq_buzz(nfmax)

      real spl_tt_fore(nfmax),
     &     spl_tt_rear(nfmax),
     &     spl_tt_buzz(nfmax)


      real dBA_bb_fore(nfmax),
     &     dBA_bb_rear(nfmax)

      real dBA_tt_fore(nfmax),
     &     dBA_tt_rear(nfmax),
     &     dBA_tt_buzz(nfmax)


      include 'freq.inc'

      data pi / 3.1415926535897932384626 /

      gamma = 1.4
      R = 287.0
      Tt2 = T0*(1.0 + 0.5*(gamma-1.0)*M0**2)

c---- Change in stagnation temperature across the fan/compressor stage
      gmig = (gamma-1.0)/gamma
      dTt = Tt2*(FPR**gmig - 1.0)/etaf

c---- mass flow through the fan
      mfan = mdot*BPR

c---- blade passing frequency in Hz
      bpf = B*RPM/60.0

c-    if set to -1 (ingested) then 6dB removed from combination tones
c-    if set to +1 (expelled) then no change made to ESDU method
      shocklocation = +1

c---- BROADBAND 
      call esdu98008broadband_total(+1,method,
     &       Dist,theta,dTt,mfan,rss,Mtr,Mtrd,bpf,
     &       nfmax,
     &       nf_bb_fore, freq_bb_fore, spl_bb_fore )
      call esdu98008broadband_total(-1,method,
     &       Dist,theta,dTt,mfan,rss,Mtr,Mtrd,bpf,
     &       nfmax,
     &       nf_bb_rear, freq_bb_rear, spl_bb_rear )

c      do i = 1, nf_bb_fore
c        write(*,*) i, spl_bb_fore(i)
c      enddo

c---- TONES
      call esdu98008discretetone_total(+1,method,
     &       Dist,theta,dTt,mfan,Mtr,Mtrd,Mt,RPM,rss,B,V,bpf,
     &       nfmax,
     &       nf_t_fore, freq_t_fore, spl_t_fore )
      call esdu98008discretetone_total(-1,method,
     &       Dist,theta,dTt,mfan,Mtr,Mtrd,Mt,RPM,rss,B,V,bpf,
     &       nfmax,
     &       nf_t_rear, freq_t_rear, spl_t_rear )

c---- BUZZSAW
      if(Mtr .gt. 1.0) then
       call esdu98008combinationtone_total(
     &         Dist,theta,shocklocation,rss,dTt,mfan,Mtr,bpf,
     &         nfmax,
     &         nf_buzz, freq_buzz, spl_buzz )

c       write(*,*) spl_buzz

      else
       nf_buzz = 0
      endif

c---- convert tones to 3rd octave
      call convert_tones_to_third_octave(
     &              nf_t_fore, freq_t_fore, spl_t_fore, 
     &              spl_tt_fore )
      call convert_tones_to_third_octave(
     &              nf_t_rear, freq_t_rear, spl_t_rear,
     &              spl_tt_rear )

      if(Mtr .gt. 1.0) then
       call convert_tones_to_third_octave(
     &              nf_buzz, freq_buzz, spl_buzz,
     &              spl_tt_buzz )
      else
       do i = 1, nfreq
         spl_tt_buzz(i) = -40.0
       enddo
      endif

c---- correction for A-weighting, atmospheric attenuation.
c      ( dBatten is dB per 100 meters )
      do i = 1, nfreq
        dBA_tt_fore(i) = spl_tt_fore(i) + 3.0
     &                 + dBAcorr(i)
     &                 + dBatten(i) * Dist/100.0
        dBA_bb_fore(i) = spl_bb_fore(i) + 3.0
     &                 + dBAcorr(i)
     &                 + dBatten(i) * Dist/100.0
        dBA_tt_fore(i) = max( dBA_tt_fore(i) , 0.0 )
        dBA_bb_fore(i) = max( dBA_bb_fore(i) , 0.0 )

        dBA_tt_rear(i) = spl_tt_rear(i) + 3.0
     &                 + dBAcorr(i)
     &                 + dBatten(i) * Dist/100.0
        dBA_bb_rear(i) = spl_bb_rear(i) + 3.0
     &                 + dBAcorr(i)
     &                 + dBatten(i) * Dist/100.0
        dBA_tt_rear(i) = max( dBA_tt_rear(i) , 0.0 )
        dBA_bb_rear(i) = max( dBA_bb_rear(i) , 0.0 )

        if(Mtr .gt. 1.0) then
          dBA_tt_buzz(i) = spl_tt_buzz(i) + 3.0
     &                   + dBAcorr(i)
     &                   + dBatten(i) * Dist/100.0
          dBA_tt_buzz(i) = max( dBA_tt_buzz(i) , 0.0 )
        else
          dBA_tt_buzz(i) = -40.0
        endif
      enddo

c      do i = 1, nf_bb_fore
c        write(*,*) i, dBA_bb_fore(i)
c      enddo



c---- Integration over the frequency range
      f_t_sum = 0.
      f_b_sum = 0.
      r_t_sum = 0.
      r_b_sum = 0.
      f_z_sum = 0.
      do i = 1, nfreq
        f_t_sum = f_t_sum + 10.0**(dBA_tt_fore(i)/10.0)
        f_b_sum = f_b_sum + 10.0**(dBA_bb_fore(i)/10.0)
        r_t_sum = r_t_sum + 10.0**(dBA_tt_rear(i)/10.0)
        r_b_sum = r_b_sum + 10.0**(dBA_bb_rear(i)/10.0)
        f_z_sum = f_z_sum + 10.0**(dBA_tt_buzz(i)/10.0)
      enddo
      spl_f_t_dba = 10.0*log10(f_t_sum)
      spl_f_b_dba = 10.0*log10(f_b_sum)
      spl_r_t_dba = 10.0*log10(r_t_sum)
      spl_r_b_dba = 10.0*log10(r_b_sum)
      spl_f_z_dba = 10.0*log10(f_z_sum)

      return
      end ! esdu_fan_noise_total


      subroutine convert_tones_to_third_octave(
     &            nf, f, spl,
     &            third_octave )
c---------------------------------------------------------------------------
c Sums tones into 1/3rd octave values
c
c freq = [1:nf] array of frequencies
c spl  = [1:nf] array of SPL tone value
c
c third_octave = [1:24] array of 1/3rd octave values for each tone.
c
c---------------------------------------------------------------------------
      implicit real (a-z)
      integer nf
      real f(nf), spl(nf)
      real third_octave(*)

      integer which_third_octave
      integer i, k

      include 'freq.inc'

      do i = 1, nfreq
        third_octave(i) = -40.0
      enddo

      do k = 1, nf
        i = which_third_octave(f(k))
        if(i .ne. 0) then
          if(third_octave(i) .eq. -40.0) then
c---------- replace line with new spl
            third_octave(i) = spl(k)
          else
c---------- add spl to current ones
            ssum = 10.0**(third_octave(i)/10.0)
     &           + 10.0**(spl(k)/10.0)
            third_octave(i) = 10.0*log10( ssum )
          endif
        endif
      enddo
      
      return
      end


      integer function which_third_octave(f)
c----------------------------------------------------------------------
c     Returns the third octave band that the frequency f is in 
c     from 1 to nfreq, or 0 if outside the preset frequency range 
c----------------------------------------------------------------------
      real f

      include 'freq.inc'

      do i = 1, nfmid
        if(f .lt. fmid(i)) then
          which_third_octave = i-1
          return
        endif
      enddo

      which_third_octave = 0
      return
      end ! which_third_octave


      subroutine esdu98008broadband_total(
     &        direc,method,
     &        Dist,theta,dTt,mfan,rss,Mtr,Mtrd,bpf,
     &        nfmax, 
     &        nf, f, spl )
c--------------------------------------------------------------------
c Estimates broadband noise from fan, rearward and forward.
c
c Returns 1/3rd octave predicted SPL values from 50Hz to 10kHz 
c   (24 in total)
c
c ONLY IMPLEMENTED FOR SINGLE STAGE FAN WITHOUT IGVs
c
c Definition of
c input parameters is given above in function esdu_fan_noise_total
c--------------------------------------------------------------------
      implicit real (a-z)

      integer direc, method, nfmax, nf

      real f(nfmax), spl(nfmax)

      integer i

      include 'freq.inc'

c---- set frequencies covering spectrum
      nf = nfreq
      do i = 1, nf
        f(i) = freq(i)
      enddo

c----------------------------------------------------------
c---- FUNCTION 1 - Mach number correlation
      if(method.eq.1 .or. method.eq.3) then
        if(direc .eq. +1) then
          L = 55.5
          M_cutover = 0.9
        else
          L = 58.0
          M_cutover = 1.0
        endif

      else
        if(direc .eq. +1) then
          L = 58.5
          M_cutover = 0.9
        else
          L = 60.0
          M_cutover = 1.0
        endif
      endif

     
      if(Mtrd .ge. 1.0) then
        if(Mtr .gt. M_cutover) then
          F1 = L + 20.0*log10(Mtrd) - 20.0*log10(Mtr)
        else
          F1 = L + 20.0*log10(Mtrd)
        endif
      elseif(Mtr .le. M_cutover) then
        F1 = L
      else
c------ correlation does not cover this 
        F1 = 0.0
      endif
      
c----------------------------------------------------------
c---- FUNCTION 2 - rotor stator spacing correlation
      F2 = -5.0 * log10(rss/300.0)

c----------------------------------------------------------
c---- FUNCTION 3 - directivity
      F3 = BBdirfun(theta,direc,method)

cc      write(*,*) 'F3', F3

c----------------------------------------------------------
c---- basic broadband noise part, same for all frequencies
      spl0 = 20.0*log10( dTt/0.555)
     &     + 10.0*log10(mfan/0.454)
     &     + F1 + F2 + F3 - 20.0*log10(Dist)

C---- add on frequency-dependent part F4, clip to zero if negative
      do i = 1, nf
        if    (method .eq. 1) then
          arg = exp( -0.35*( log(f(i)/(2.0*bpf))/log(2.2) )**2 )
        elseif(method .eq. 2) then
          arg = exp( -0.50*( log(f(i)/(2.5*bpf))/log(2.2) )**2 )
        else
          if(bpf .gt. 0.5*f(i)) then
            arg = exp( -0.35*( log(f(i)/(2.0*bpf))/log(2.2) )**2 )
          else
            arg = exp( -2.00*( log(f(i)/(2.0*bpf))/log(2.2) )**2 )
          endif
        endif
        F4 = 10.0*log10(arg)

c       write(*,*) 'F4', F4

        spl(i) = max( spl0 + F4 , 0.0 )
      enddo

      return
      end ! esdu98008broadband_total


      subroutine esdu98008discretetone_total(
     &        direc,method,
     &        Dist,theta,dTt,mfan,Mtr,Mtrd,Mt,RPM,rss,B,V,bpf,
     &        nfmax,
     &        nf, f, spl )
c----------------------------------------------------------------------
c This function estimates tonal noise from fan, forward and rearward
c
c Returns 1/3rd octave predicted SPLs from 50Hz to 10kHz (24 in total)
c
c ONLY IMPLEMENTED FOR SINGLE STAGE FAN WITHOUT IGVs
c
c Definition of  input parameters is given above in esdu_fan_noise_total
c----------------------------------------------------------------------
      implicit real (a-z)
      integer direc, method, nfmax, nf
      real f(nfmax), spl(nfmax)

      integer i
      include 'freq.inc'

c----------------------------------------------------------------
c---- FUNCTION 1 - mach number correlation
      if(method.eq.1 .or. method.eq.2) then
        if(direc.eq.+1) then
c-------- forward
          L = 54.5
          M_cutover = 0.72
        else
c-------- backward
          L = 59.0
          M_cutover = 1.0
        endif
      else
        if(direc.eq.+1) then
c-------- forward
          L = 60.5
          M_cutover = 0.72
        else
c-------- backward
          L = 63.0
          M_cutover = 1.0
        endif
      endif

      if(Mtrd .ge. 1.0) then
        if(Mtr .gt. M_cutover) then
          if(direc.eq.+1) then
            L1 = L + 20.0*log10(Mtrd) + 50.0*log10(Mtr/0.72)
            L2 = L + 20.0*log10(Mtrd) + 80.0*log10(Mtrd/Mtr) - 1.0
            F1 = min( L1 , L2 )
          else
            F1 = L + 20.0*log10(Mtrd) - 20.0*log10(Mtr)
          endif
        else
          F1 = L + 20.0*log10(Mtrd)
        endif
      elseif(Mtr .le. M_cutover) then
          F1 = L
      else
c---- corellation does not cover this 
       write(*,*) '? esdu98008discretetone_total:', Mtrd,Mtr,M_cutover
       stop
      endif

c----------------------------------------------------------------
c---- FUNCTION 1 - rotor stator spacing correlation
      F2 = -10.0 * log10(rss/300.0)

c----------------------------------------------------------------
c---- FUNCTION 3 - directivity 
      F3 = DTdirfun(theta,direc,method)

c      write(*,*) 'F3', F3

c----------------------------------------------------------------
c---- TOTAL for the fundamental tone
      Lp = 20.0*log10(dTt/0.555)
     &   + 10.0*log10(mfan/0.454)
     &   + F1 + F2 + F3 - 20.0*log10(Dist)

      Lp = max( Lp , 0.0 )


c----------------------------------------------------------------
c---- SPL for all harmonic tones <= 10kHz and one more
      cut_off_factor = abs( Mt/(1.0 - V/B) )

c---- fundamental blade-passing frequency
      f(1) = bpf
      nf = 1

c---- 2x, 3x, 4x... harmonics <= 10kHz 
      do i = 1, nfmax-1
c------------------------------------------------------------------
c------ FUNCTION 4 - frequency A
        if(method.eq.1 .or. method.eq.3) then
          if    (i.eq.1) then
            if(cut_off_factor .lt. 1.05) then
              F4 = Lp - 8.0    
            else
              F4 = Lp
            endif
          elseif(i.eq.2) then
            F4 = Lp - 9.2
          else
            F4 = Lp - 3.0*float(i) - 1.8
          endif
        else
          if(i.eq.1 .and. cut_off_factor .lt. 1.05) then
            F4 = Lp - 8.0
          else
            F4 = Lp + 3.0 - 3.0*float(i)
          endif
        endif
  
c----------------------------------------------------------------
c------ FUNCTION 5 - frequency B
        F5 = 0.0

c----------------------------------------------------------------
c------ calculate spl for freq
        if(direc .eq. +1) then
         spl(i) = 10.0 * log10(10.0**(F4/10.0) + 10.0**(F5/10.0))
        else
         spl(i) = F4
        endif


        if(f(i) .ge. 10000.0) then
c-------- over threshold... jump out of frequency loop
          go to 10
        else
c-------- set new harmonic frequency
          f(i+1) = f(i) + bpf
          nf = i+1
        endif

      enddo
    
 10   continue

      end ! esdu98008discretetone_total


      subroutine esdu98008combinationtone_total(
     &         Dist,theta,shocklocation,rss,dTt,mfan,Mtr,bpf,
     &         nfmax,
     &         nf, f, spl )
c----------------------------------------------------------------------
c This function estimates the "buzz-saw"noise when the fan tip is supersonic
c
c Returns predicted SPL values for combination tones (BUZZSAW) 
c for 1/2, 1/4, 1/8 BPF for polar angle theta
c
c ONLY IMPLEMENTED FOR SINGLE STAGE FAN WITHOUT IGVs
c
c Definition of
c input parameters is given above in fucntion esdu_fan_noise_total
c----------------------------------------------------------------------
      implicit real (a-z)

      integer shocklocation, nfmax, nf

      real f(nfmax), spl(nfmax)

c---------------------------------------------------------------
c---- FUNCTION 1 - Mach number correlation
      if(Mtr .lt. 1.135) then
        F1a = 50.0 + 6.1*(Mtr-1.0  )/0.135
        F1b = F1a
      else
        F1a = 56.1 - 3.1*(Mtr-1.135)/0.865
        F1b = F1a
      endif
      if(Mtr .lt. 1.413) then
        F1c = 40.0 + 7.3*(Mtr-1.0  )/0.413
      else
        F1c = 47.3 - 3.1*(Mtr-1.413)/0.587
      endif


c---------------------------------------------------------------
c---- FUNCTION 2 - directivity
c      t = theta
c      F2 = -5.553
c     &    + 5.396*cos(  t)
c     &    - 2.236*cos(2*t)
c     &    - 1.313*cos(3*t)
c     &    - 1.032*cos(4*t)

        F2 = BZdirfun(theta)


c---------------------------------------------------------------
c---- FUNCTION 3 - correction for shock ingestion
      if    (shocklocation.eq.-1) then
       F3 = -6.0
      elseif(shocklocation.eq.+1) then
       F3 = 0.0
      else
       write(*,*) 'Shock location not recognised:', shocklocation
      endif


c---------------------------------------------------------------
c---- add up for three frequencies
      nf = 3
   
      spl(1) = 20.0*log10(dTt /0.555)
     &       + 10.0*log10(mfan/0.454)
     &       + F1a + F2 + F3
     &       - 20.0*log10(Dist)
      spl(2) = 20.0*log10(dTt /0.555)
     &       + 10.0*log10(mfan/0.454)
     &       + F1b + F2 + F3
     &       - 20.0*log10(Dist)
      spl(3) = 20.0*log10(dTt /0.555)
     &       + 10.0*log10(mfan/0.454)
     &       + F1c + F2 + F3
     &       - 20.0*log10(Dist)

      spl(1) = max( 0.0 , spl(1) )
      spl(2) = max( 0.0 , spl(2) )
      spl(3) = max( 0.0 , spl(3) )

      f(1) = bpf/2.0
      f(2) = bpf/4.0
      f(3) = bpf/8.0

      end ! esdu98008combinationtone_total



      real function fsfun(Arat,Vrat)
c--------------------------------------------------------------------
c  Representation of Figure 2 from AIAA JOURNAL. Vol.21, No.3. p.337
c  Recommended frequency shift parameter for coaxial jets 
c  with respect to isolated primary jet.
c  To calculate spectra use relation A2 from paper (S parameter)
c
c  Arat   jet area ratio      A2/A1
c  Vrat   jet velocity ratio  V2/V1
c
c  fsfun  frequency shift parameter
c--------------------------------------------------------------------
      implicit real (a-z)

      Apar = log10(1.0 + Arat)
      w = 1.0 - abs(Vrat-0.64)**1.1
      fsfun = 1.0 - exp(-1.28*Apar*w)

      return
      end ! fsfun


      real function fpfun(S,t)
c--------------------------------------------------------------------
c  Representation of Table 1 from AIAA JOURNAL. Vol.21, No.3. p.337
c  Recommended spectra for jet mixing noise.
c
c  S   effective Strouhal number
c  t   corrected directivity angle   [deg]
c
c  fpfun  normalized sound pressure level SPL - UOL,  [dB]
c--------------------------------------------------------------------
      implicit real (a-z)

      Sl = max( -3.6 , min( 3.6 , log10(S) ) )
      tl = max( 110.0, min( 250.0 , t ) )

      Slo = -0.3 - 0.012*(tl-110.0) + 0.00003*(tl-110.0)**2;
      
      ap =  12 + 0.00015*(tl-110)**3 / (1 + 0.0000115*(tl-110)**3);
      am = -35 + 0.007*(tl-157)**2 / (1.0 + 0.00022*(tl-157)**2);

      fo = -2.5 - 0.0022*(tl-120)**2/(1 + 0.000027*(tl-120)**2);

      w = 0.14 - 0.07 / (1 + 0.00005*(tl-110)**3);

      fpfun = -log(exp(w*am*(Sl-Slo)) + exp(w*ap*(Sl-Slo)))/w  +  fo

      return
      end ! fpfun


      real function dBAcorrf(f)
c---------------------------------------------------
c     Returns A-weighting correction in dB
c     for specified frequency f [Hz]
c---------------------------------------------------
      implicit real (a-z)

c---- exact form
      den1 = f**2 + 20.6**2
      den2 = sqrt((f**2 + 107.0**2)*(f**2 + 737.9**2))
      den3 = f**2 + 12200.0**2
      RA = 12200.0**2 * f**4 / (den1*den2*den3)
      dBAcorrf = 2.0 + 20.0*log10(RA)

cc---- much simpler form, only slightly different from above
c      dBAcorr = 1.3 - 10.2*( log10(f/2450.0) )**2.15

      return
      end ! dbcorr


      real function dBattenf(f)
c---------------------------------------------------
c     Returns atmospheric attenuation in dB / 100m
c     for specified frequency f [Hz]
c---------------------------------------------------
      implicit real (a-z)
      
      den2 = 1 + 0.086*(f/1000.0)**2
      den1 = 1 + 1.46*(f/1000.0)**2 / den2
      dBattenf = -1.53*(f/1000.0)**2 / den1

      return
      end ! dBatten



      real function BBdirfun(theta,direc,method)
c---------------------------------------------------
c     Returns broadband directivity factor versus angle theta
c---------------------------------------------------
      implicit real (a-z)
      integer direc, method

      integer k
      real a(7,3)
      data a /
     &  -21.319, +27.786,  -7.272, +1.155, -1.604, +0.371, -0.794,
     &  -11.600,  -8.581,  -9.443, +1.443, -2.134, +0.793, -0.751,
     &  -13.761, -12.442, -10.418, +1.761, -2.442, +0.487, -0.943  /

      t = theta

      if(direc .eq. 1) then
        k = 1
      else
        if(method.eq.1 .or. method.eq.3) then
         k = 2
        else
         k = 3
        endif
      endif

      BBdirfun = a(1,k)
     &         + a(2,k)*cos(    t)
     &         + a(3,k)*cos(2.0*t)
     &         + a(4,k)*cos(3.0*t)
     &         + a(5,k)*cos(4.0*t)
     &         + a(6,k)*cos(5.0*t)
     &         + a(7,k)*cos(6.0*t)

      return
      end ! BBdirfun


 
      real function DTdirfun(theta,direc,method)
c---------------------------------------------------
c     Returns discrete tone directivity factor versus angle theta
c---------------------------------------------------
      implicit real (a-z)
      integer direc, method

      integer k
      real a(7,4)
      data a /
     &  -13.875, +17.112, -5.258, +0.501, -0.748, +0.574, -0.457,
     &  -20.917, +27.563, -7.531, +0.935, -1.696, +0.416, -0.698,
     &   -9.917, -10.048, -9.458, +0.515, -1.809, +0.519, -1.034,
     &  -13.056, -12.236, -9.685, +1.456, -2.258, +0.388, -0.625  /

      t = theta

      if(direc .eq. 1) then
        if(method.eq.1 .or. method.eq.3) then
         k = 1
        else
         k = 2
        endif
      else
        if(method.eq.1 .or. method.eq.3) then
         k = 3
        else
         k = 4
        endif
      endif

      DTdirfun = a(1,k)
     &         + a(2,k)*cos(    t)
     &         + a(3,k)*cos(2.0*t)
     &         + a(4,k)*cos(3.0*t)
     &         + a(5,k)*cos(4.0*t)
     &         + a(6,k)*cos(5.0*t)
     &         + a(7,k)*cos(6.0*t)

      return
      end ! DTdirfun

 
      real function BZdirfun(theta)
c---------------------------------------------------
c     Returns fan buzz directivity factor versus angle theta
c---------------------------------------------------
      implicit real (a-z)
c      integer direc, method

      real a(5)
      data a / -5.553, +5.396, -2.236, -1.313, -1.032 /

      t = theta

      BZdirfun = a(1)
     &         + a(2)*cos(    t)
     &         + a(3)*cos(2.0*t)
     &         + a(4)*cos(3.0*t)
     &         + a(5)*cos(4.0*t)

      return
      end ! BZdirfun

