
      subroutine wsize(pari,parg,parm,para,pare,
     &                 iterwmax, initeng,iairf,
     &                 ichoke5,ichoke7,
     &                 Litprint,Lconv)
c=====================================================================
c     Sizes aircraft and calculates performance for design mission
c     
c     Input:
c      pari(.)    flags
c      iterwmax   max number of weight/sizing iterations
c      initeng    0 = initialize engine states for engine calcs
c                 1 = use existing states to start engine calcs
c      iairf      index of airfoil database to use
c      Litprint   T = print weight-iteration history
c
c     Input/Output:
c      parg(.)   geometry parameters
c      parm(.)   mission  parameters
c      para(.p)  aero     parameters for points p=1..iptotal
c      pare(.p)  engine   parameters for points p=1..iptotal
c
c     Output:
c      ichoke5(p)  0 = core nozzle unchoked, 1 = core nozzle choked
c      ichoke7(p)  0 = fan  nozzle unchoked, 1 = fan  nozzle choked
c      Lconv       T = convergence successful
c=====================================================================
      implicit real (a-h,l-z)
c
      ip = ipcruise1
      call fusebl(pari,parg,para(1,ip))
      DAfsurf = para(iaDAfsurf,ip)
      DAfwake = para(iaDAfwake,ip)

c------------------------------------------------------------------------
c---- set specified quantities which are fixed during weight iteration loop
c---- parameters for sizing mission
c---- store design-mission parameters in geometry array
c---- fixed weight and location
c---- weight fractions 
c---- fuselage lift carryover loss, tip lift loss factors
c---- fuselage dimensions and coordinates
c---- payload-proportional weights
c---- window and insulation densities per length and per area
c---- fuselage-bending inertial relief factors
c---- tail CL's at structural sizing cases
c---- wing break, wing tip taper ratios
c---- tail surface taper ratios (no inner panel, so lambdas=1)
c---- tailcone taper ratio
c---- wing geometry parameters
c---- tail geometry parameters
c---- number of vertical tails
c---- strut vertical base height, h/c, strut shell t/h
c---- assume no struts on tails
c---- max g load factors for wing, fuselage
c---- never-exceed dynamic pressure for sizing tail structure
c---- wingbox stresses and densities
c---- fuselage stresses and densities
c---- fuselage shell bending/skin modulus ratio
c---- strut stress and density
c---- assume tail stresses and densities are same as wing's (keeps it simpler)
c---- number of engines, y-position of outermost engine
c---- fan hub/tip ratio
c---- nacelle wetted area / fan area ratio

c------------------------------------------------------------------
c---- set known operating conditions at T/O altitude
c---- interpolate CL over climb points,   ipclimb1+1 ... ipclimbn
c---- interpolate CL over descent points, ipdescent1 ... ipdescentn-1
c---- initialize fan duct conditions for initial Cf calculation
c---- estimate takeoff speed and set V,Re over climb and descent

c==========================================================================
c---- initial-guess section, to allow starting weight iteration loop
c---- wing panel weights and moments (estimate span first)
c---- HT lift
c---- store initial weight guesses
c---- wing centroid x-offset from wingbox
c---- tail area centroid locations (assume no offset from sweep initially)
c---- center wing box chord extent for fuselage weight calcs (small effect)
c---- nacelle, fan duct, core cowl lengths Re calculations
c---- nacelle Awet/S
c---- fan flow Awet/S
c---- estimate fuel fraction from Breguet
c---- mission-point fuel fractions
c---- max fuel fraction is at start of climb
c---- clear climb angles, to force initial guesses
c---- put initial-guess weight fractions in mission-point array
c---- initial tail info needed for initial fuselage bending and torsion material
c---- wing and tail total pitching moments (including sweep)
c---- initial cruise-climb angle gamVcr needed to estimate end-of cruise altitude,
c-     to set initial cabin-pressure deltap for sizing fuselage shell
c---- pressure and altitude at start of cruise 
c-     (this won't change -- it's needed for end of cruise stuff below)
c---- guess pressure p0d at end-of-cruise (scales with weight),
c-     for initial cabin delta(p)
c---- guess for single-engine thrust, speed, fan area, for engine-out VT sizing
c---- guess for fan-face Mach numbers, for nacelle CD calculation
c---- calculate initial guesses for cooling mass flow ratios epsrow(.)
c---- initialize previous-iteration weights (none yet)

c==============================================================================
c---- top of weight-iteration loop
      do 100 iterw=1, iterwmax

c-------------------------------------------------------------------------
c---- fuselage sizing section
c---- max tail lifts at maneuver dynamic pressure qne
c---- max deltap (fuselage overpressure) at end of cruise-climb, assumes p ~ W/S
c---- engine weight, if any, mounted on tailcone
c---- fuselage weight, with skin sized by max deltap, tail loads sized by qne
c---- buoyancy weight needed to size wing and engine at cruise

c-------------------------------------------------------------------------
c---- total max takeoff weight
c---- this calculated WMTO is the design-mission WTO

c-------------------------------------------------------------------------
c---- convergence tests
c---- set max error from last 2 iterations 
c-    (prevents false convergence from "lucky" near-zero single change)

c----------------------------------------------------------------------------
c---- wing sizing section
c---- size wing area and chords at start-of-cruise
c---- update center wing box chord for FUSEW on next cycle
c---- wing centroid x-offset from wingbox
c---- set wing pitching moment constants
c---- set wing center load po using cruise spanload cl(y)
c---- size wing structure and weights
c---- strut chord (perpendicular to strut)
c---- individual panel weights

c----------------------------------------------------------------------------
c---- tail sizing section
c---- set tail CL derivative fraction
c---- set nacelle CL derivative fraction
c----- size horizontal tail and locate wing simultaneously
       call htsize(pari,parg,

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c---- set HT max loading magnitude

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c---- set VT max loading magnitude, based on single tail + its bottom image
c---- set actual span of single VT (not including its image)

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c---- size HT primary structure and weight
c---- HT area centroid x-offset from box
c---- set HT piching moment constants

c- - - - - - - - - - - - - - - - - - - - - -
c---- size VT (+ its image) primary structure and weight
c---- set actual VT tail weights (half of tail+image, times number of tails)
c---- VT area centroid x-offset from box

c-------------------------------------------------------------------
c---- might as well recalculate max weight with latest weight info

c-------------------------------------------------------------------
c---- engine and nacelle sizing and weight section

c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c---- calculate stuff for start-of-cruise (cruise1) point
c---- set pitch trim by adjusting CLh, to get correct wing cl for CDSUM
      call balance(pari,parg,para(1,ip),rfuel,rpay,xipay, 

c---- calculate overall CD
c---- size engine for cruise1 point
c---- store design-point parameters for all operating points
c---- bare weight for one engine [Newtons]

c- - - - - - - - - - - - - - - - - - - - - - - - - - - 
c----- Drela's original weight model
c- - - - - - - - - - - - - - - - - - - - - - - - - - - 
c------ Nate Fitzgerald's weight model, ungeared fan
c------- basic technology
c------- advanced technology
c- - - - - - - - - - - - - - - - - - - - - - - - - - - 
c------ Nate Fitzgerald's weight model, geared fan
c------- basic technology
c------- advanced technology

c- - - - - - - - - - - - - - - - - - - - - - - - - - - 
c---- bare engine weight
c---- nacelle weight model from NASA CR 151970

c- - - - - - - - - - - - - - - - - - - - - - - - - - - 
c---- engine accessories + fuel system
c---- engine pylons
c---- total engine weight

c- - - - - - - - - - - - - - - - - - - - - - - - - - - 
c---- set new nacelle area / reference area  fraction fSnace

c-------------------------------------------------------------------
c---- aero calculations and mission-simulation section
      call mission(pari,parg,parm,para,pare,

c---- this calculated fuel is the design-mission fuel 
c-------------------------------------------------------------------
c---- size cooling mass flow at takeoff rotation condition (at Vstall)
c---- must define CDwing for this point in case there's wing BLI
c---- Tmetal was specified... set cooling flow ratios for all points

c-----------------------------------------------------------------
c---- recalculate max weight with latest weight info

c-----------------------------------------------------------------
c---- set previous-iteration weights for next iteration

 100  continue
c=======================================================================
c---- jump to here if converged successfully

c-----------------------------------------------------------------
c---- normal takeoff and balanced-field takeoff calculations
c---- set static thrust for takeoff routine
c---- set rotation thrust for takeoff routine
c-     (already available from cooling calculations)
c---- calculate takeoff and balanced-field lengths
c---- calculate noise
c---- calculate CG limits from worst-case payload fractions and packings

c---- set neutral point
      call balance(pari,parg,para(1,ip), rfuel,rpay,xipay, 

      end ! wsize

